function [results]=automatic(grocer_namey0,varargin)

// PURPOSE: automatic selection of a model by least-squares
// regressions as in Hendry and Krozlic: starting from the 
// list of exogenous variables provided by the user, the 
// function select the "best" regression, that is one where
// all variables are significant, all specification tests are
// passed, and, if more than one regression has these
// properties, the one with the smallest aic criterion
// ------------------------------------------------------------
// REFERENCES:
// Krolzig, H.-M. and Hendry, D.F. (2001): "Computer Automation
//  of General-to-Specific Model Selection Procedures", Journal
// of Economic Dynamics and Control, 25 (6-7), 831-866. 
// ------------------------------------------------------------
// INPUT: 
// * grocer_namey0 = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'prt=opt1,opt2,...,optn' 
//   where opti is one of the following available options:
//     - nothing (nothing printed !)
//     - initial (results of initial model printed)
//     - st0_mod (results of stage 0 model printed)
//     - st1_mod (results of stage 1 models printed)
//     - st1_union (results of stage 1 union model printed)
//     - st2_mod (results of stage 2 models printed)
//     - st2_union (results of stage 2 union model printed)
//     - final (results of final model printed)
//     - test_inter (results of specification tests for the
//       intermediate results printed)
//     - test_final (results of specification tests for the
//       final model printed)
//     - test (specification tests printed for every regression
//       result printed)
//     - st1_path (results of stage 1 paths printed)
//     - st2_path (results of stage 2 paths printed)
//     - all (all results printed: can be very long !) 
//   . the string 'test=opt1,opt2,...,optn'
//   where opti is one of the following available options:
//     - chowtest(p)
//     - predfailin(p)
//     - doornhans
//     - bpagan(p)
//     - hetero_sq
//   . the string 'alpha=p' for the simplification significance
//     level (default = 0.01)
//   . the string 'f0_sig=p' for the pre-test (default = 0.9)
//   . the string 'eta=p' (default = 0.01)
//   . the string 'gam=p' (default = 0.05)
//   . the string 'crit=x' where x= aic, bic, or hq(default bic)
//   . the string 'test=x1,...,xp' where xi is the name of a
//     test function 
//   . the string 'newname(x1,x2)' where x1 is the name of a
//     test function and x2 is the corresponding name the user
//     wants the program to display
// ------------------------------------------------------------
// OUTPUT: 
// a results tlist with the following fields:
// * results('meth') = 'automatic';
// * results('f_test') = the function used to perform the
//                      specification tests
// * results('ending reason') = the reason why the final model 
//                            has been chosen
// * results('initial model') = the estimation results of the
//                            unrestricted model
// * results('stage 0 model') = the estimation results of the 
//                             stage 0 restricted model (all
//                             variables whose individual tstat
//                             level is lower than 1 and whose
//                             joint significance level is
//                             lower than f0_sig are withdrawn) 
// * results('final model') = the estimation results of the
//                            final model
// * results('stage i models') = the estimation results of the
//                            stage i (i=1,2) models and the 
//                            corresponding paths
// * results('stage i union model') = the estimation results of
//                            the model built from the union of  
//                            stage i (i=1,2) models   
// ------------------------------------------------------------
// PRINTS: results along the options given by the user
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002


// set default values
grocer_alpha=0.05
grocer_gam=0.05
grocer_f0_sig=0.9
grocer_descent=3
grocer_lprt=emptystr()
grocer_ltest=emptystr()
grocer_descent=5
grocer_crit='bic'
done=%f
grocer_names=tlist(['names';'jbnorm';'doornhans';'hetero_sq';...
     'chowtest';'predfailin';'arlm'],'Jarque & Bera',...
     'Doornik & Hansen','hetero x_squared','Chow',...
     'Chow pred. fail. ','AR')

// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[grocer_y,grocer_namey,grocer_prests,grocer_boundsvarb]=...
exploy(grocer_namey0)

grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      varargin(grocer_i)=strsubst(varargin(grocer_i),' ','')
      str3=part(varargin(grocer_i),[1:3])
      str4=part(varargin(grocer_i),[1:4])
      str5=part(varargin(grocer_i),[1:5])
      str7=part(varargin(grocer_i),[1:7])
      if str3 == 'eta' | str3 == 'gam' | str7 == 'st0_sig'...
        | str7 == 'descent' | str5 == 'alpha' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str4 == 'test' then
         grocer_ltest=varargin(grocer_i)
         varargin(grocer_i)=null()
      elseif str4 == 'crit' then
         grocer_crit=part(varargin(grocer_i),...
           strindex(varargin(grocer_i),'=')+1:length(varargin(grocer_i)))
         if grocer_crit ~= 'aic' & grocer_crit ~= 'bic' &...
           grocer_crit ~= 'hq' then
            error('not an available information criterion: '+grocer_crit)
         end
         varargin(grocer_i)=null()
      elseif str3 == 'prt' then
         grocer_lprt=varargin(grocer_i)
         varargin(grocer_i)=null()
      elseif str7 == 'newname' then
         indcom=strindex(varargin(grocer_i),',')
         grocer_names(1)($+1)=part(varargin(grocer_i),...
         strindex(varargin(grocer_i),'(')+1:indcom-1) 
         grocer_names($+1)=part(varargin(grocer_i),...
         indcom+1:strindex(varargin(grocer_i),')')-1)
         varargin(grocer_i)=null()
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' &...
   typeof(varargin(grocer_i)) ~= 'ts' then
      error('wrong type for entry number '+string(grocer_i+2))
   end
end

grocer_lx=varargin
// explode the list of the arguments into the corresponding
// variable, its name, and, if necessary updates the bounds
[grocer_x,grocer_namexos,grocer_boundsvarb]=...
explox(grocer_lx,'exogenous',grocer_boundsvarb)

// if necessary, define y on the admissible bounds
if typeof(grocer_y) == 'ts' then
   grocer_y=ts2vec(grocer_namey0,grocer_boundsvarb)
else
   if typeof(grocer_y) ~= 'constant' then
      error('variable '+grocer_namey+' is neither a ...
            time series nor a constant vector')
   end
end

[nobs,nvar]=size(grocer_x)
[test_func,m2prt_test,nvarmax]=auto_test(grocer_names,grocer_ltest,nobs,nvar)

[nobs,nvar]=size(grocer_x)
if nvar >= nobs then
   error('too many exogenous variables')
end

if nvar >= nvarmax then
   error('too many exogenous variables')
end


prescte=%f
indcte=1
while (indcte <= nvar) & ~prescte then
   prescte=and(grocer_x(:,indcte)==grocer_x(1,indcte))
   indcte=indcte+1
end

// define basic results tlists and update indcte
[lrmod,indcte]=def_results(grocer_y,grocer_namey,nobs,...
               grocer_prests,prescte,indcte,...
               grocer_boundsvarb,m2prt_test)

r1_00=ols2a(grocer_y,grocer_x,prescte,lrmod(2))
r1_00('namex')=grocer_namexos
[val,p]=test_func(r1_00)
r1_00('spec_test')=[val p]

if ~exists('grocer_eta','local') then
   grocer_eta=0.01*ones(size(p,1),1)
else
   grocer_eta=vec2col(grocer_eta)
end

for i=1:size(p,1)
   if p(i) < grocer_eta(i) then
// must adjust the size level of the corresponding 
// specification test  
      if p(i) > %eps*grocer_descent then
         grocer_eta(i)=p(i)/grocer_descent
      else
// the adjustment would lead to a level lower than
// scilab precision ; so set it to 0
         grocer_eta(i)=0
      end
      disp(' ')
      disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ')
      warning('level of specification test '+ m2prt_test(i)+' had ...
           to be ajusted')
      disp('a) this is perhaps the sign of a missing variable in your model') 
      disp('b) it could be worthwhile to check what happens if you run the ')
      disp('programm without the corresponding specification test')
      disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ')
   end
end

[done,r1_0,indpvalue1_00] = auto_stage0(grocer_y,grocer_x,...
                            indcte,r1_00,lrmod(1),grocer_alpha,...
                            grocer_f0_sig,grocer_eta)
results=tlist(['results';'meth';'f_test';'m_test';...
               'initial model';'ending reason';'final model'...
                ],'automatic',test_func,m2prt_test,r1_00,[],[]) 

if done then
   results('final model')=r1_00
   results('ending reason')='initial model=final model'
else
   results(1)($+1)='stage 0 model'
   results($+1)=r1_0
   listeq1=list()
   [gpvalue,indpvalue1_0]=sort(r1_0('pvalue'))
   nonsignif=sum(gpvalue > grocer_alpha)

   for i=1:nonsignif
      firstelim=indpvalue1_0(i)
      listeq1=auto_stage1(grocer_y,grocer_x,listeq1,...
              indpvalue1_00,indcte,firstelim,r1_00,lrmod(1),...
              grocer_alpha,grocer_eta)
   end

   sizelst1=length(listeq1)/3
   if sizelst1 == 0 then
// all remaining variables where significant or withdrawing 
// any of the significant variable leads to rejection of
// specification tests
      results('ending reason')='no model at the end at stage 1'
      results('final model')=r1_0
   else
      vaux=3*[1:sizelst1]-2
      results(1)($+1)='stage 1 paths'
      results($+1)=list(listeq1(vaux))

      sizel=length(listeq1)/3
      list1_2=listeq1
      for i=sizel:-1:1
         if typeof(list1_2(3*i)) ~= 'results' then
            for j=0:2
               list1_2(3*i-j)=null()
            end
         end
      end

      results(1)($+1)='stage 1 models'
      results($+1)=list1_2
 
      sizelst1=length(list1_2)/3
      vaux=3*[1:sizelst1]-1
      listx1_2=list(list1_2(vaux))
      nbmodst1=length(listx1_2)

      if nbmodst1 == 1 then
         results('ending reason')='only one model selected by ...
                                stage 1'
         results('final model')=list1_2(3)
      else
         numx3=fuslist(listx1_2)
         prescte=or(numx3 == indcte)
         r1_3=ols3a(grocer_y,grocer_x,grocer_namexos,numx3,...
           prescte,lrmod(prescte+1))
         [val,p]=test_func(r1_3)
         r1_3('spec_test')=[val p]

         results(1)($+1)='stage 1 union model'
         results('stage 1 union model')=r1_3
         vaux=3*[1:sizelst1]
         listr1_3=list(list1_2(vaux))
         accept=~zeros(nbmodst1,1)
         for i=1:nbmodst1
            if listr1_3(i)('nvar') == r1_3('nvar') then
         // selected model = union model ; cannot perform
         // wald test, but must accept selected model !
               accept(i) = %t
            else
               [fstat,fp]=waldf0(listr1_3(i),r1_3)
               accept(i) = (fp>grocer_gam)
            end
         end
         [accept,ordaccept] = sort(bool2s(accept))
         sumaccept=sum(accept)
         select sumaccept


// the following lines have been written to follow Hednry & 
// Krozlig 1999 paper, but lead sometimes to unsatisfactory
//  results, so I have put them as commentary; feel free to
// add them again !
//      case 0 then
// all terminal models are rejected
//         results('ending reason')='all stage 1 models ...
//                                rejected against their union'
//         results('final model')=r1_3
         case 1 then
         // only one terminal model is not rejected:
         // it is the final model 
            results('ending reason')='only one stage 1 model ...
                              non rejected against their union'
            results('final model')=list1_2(3*ordaccept(1))
         else
         // more than one model is accepted
            indx2_00=ordaccept(1:sumaccept)
            vaux=3*ordaccept-1
            listr2_00=list(list1_2(vaux))
            listx2_00=fuslist(listr2_00)
            indcte2=find(listx2_00 == indcte)
            prescte2=or(listx2_00 == indcte)
            x2_00=grocer_x(:,listx2_00)
            r2_00=ols3a(grocer_y,grocer_x,grocer_namexos,listx2_00,prescte2,...
               lrmod(prescte2+1))
            [val,p]=test_func(r2_00)
            r2_00('spec_test')=[val p]
            results(1)($+1)='stage 2.0 model'
            results($+1)=r2_00

            [done,r2_0,indpvalue2_00] = auto_stage0(grocer_y,...
                x2_00,indcte2,r2_00,lrmod(1),grocer_alpha,...
                grocer_f0_sig,grocer_eta)

            listeq2=list()
            [gpvalue2,indpvalue2_0]=sort(r2_0('pvalue'))
            nonsignif=sum(gpvalue2 > grocer_alpha)

            for i=1:nonsignif
               firstelim=indpvalue2_0(i)
               listeq2=auto_stage1(grocer_y,x2_00,listeq2,...
                 indpvalue2_00,indcte2,firstelim,r2_00,lrmod(1),...
                 grocer_alpha,grocer_eta)
            end
            for i=1:length(listeq2)/3
               listeq2(3*i-1)=listx2_00(listeq2(3*i-1))
            end

            sizel2=length(listeq2)/3
            list2_2=listeq2
            for i=sizel2:-1:1
               if typeof(listeq2(3*i)) ~= 'results' then
                  for j=0:2
                     list2_2(3*i-j)=null()
                  end
               else
                  list2_2(3*i)('namex')=grocer_namexos(list2_2(3*i-1))
               end
            end

            results(1)($+1)='stage 2.1 models'
            results($+1)=list2_2

            nbmodst2=length(list2_2)/3

            select nbmodst2 

            case 0 then
               if r2_00('nvar') == r2_0('nvar') then
                  results('ending reason')='no variable can be withdrawn in stage 1 union model '
               else
                  results('ending reason')='all variables significant in stage 2.0 model '
               end
               results('final model')=r2_0

            case 1 then
               results('ending reason')='only one model selected by stage 2'
               results('final model')=list2_2(3)
               results(1)($+1)='stage 2 models'
               results($+1)=list2_2

            else
               vaux=3*[1:nbmodst2]-1
               listx2_2=list(list2_2(vaux))
               numxd=fuslist(listx2_2)
               prescte=or(listx2_2 == indcte)
               r2_3=ols3a(grocer_y,grocer_x,grocer_namexos,...
                 numxd,prescte,lrmod(prescte+1))
               [val,p]=test_func(r2_3)
               r2_3('spec_test')=[val p]
               results(1)($+1)='stage 2 union model'
               results('stage 2 union model')=r2_3

               vaux=3*[1:nbmodst2]
               listr2_3=list(list2_2(vaux))
               accept=~zeros(nbmodst2,1)
               for i=1:nbmodst2
                  [fstat,fp]=waldf0(listr2_3(i),r2_3)
                  accept(i) = (fp>grocer_gam)
               end
               [accept,ordaccept] = sort(bool2s(accept))
               sumaccept=sum(accept)

               select sumaccept
               case 0 then
              // all terminal models are rejected
                  results('ending reason')='all stage 2 models rejected ...
                                    against their union'
                  results(1)($+1)='stage 2 models'
                  [val,p]=test_func(r2_3)
                  results($+1)=list([],[],r2_3,[val p])
                  results('final model')=r2_3
               
               case 1 then
            // only one terminal model is not rejected:
            // it is the final model 
                  results('ending reason')='only one stage 2 model non ...
                                        rejected against their union'
                  results('final model')=list2_2(3*ordaccept(1))
//list(list2_2(3*ordaccept(1)-2),...
//                                           list2_2(3*ordaccept(1)-1),...
//                                           list2_2(3*ordaccept(1)))
               else
            // more than one model are accepted
                  results(1)($+1)='stage 2 models'
                  crit0=%inf
                  lst2_mod=list()
                  for i=1:sumaccept
                     r=list2_2(3*ordaccept(i))
                     crit=r(grocer_crit)
                     if crit < crit0 then
                        indmof=i
                        crit0=crit
                     end
                     lst2_mod($+1)=list2_2(3*ordaccept(i)-2)
                     lst2_mod($+1)=list2_2(3*ordaccept(i)-1)
                     lst2_mod($+1)=list2_2(3*ordaccept(i))
                  end
                  results('stage 2 models')=lst2_mod
                  results('ending reason')='stage 2 models selected by '...
                     +grocer_crit+' criterion'
                  results('final model')=list2_2(3*ordaccept(indmof))
               end       
            end
         end
      end
   end
end

// print the results
prtauto(results,grocer_lprt,%io(2))

