function [resadf]=adf(grocer_namey,grocer_p,grocer_l,grocer_np)

// PURPOSE: carry out augmented Dickey-Fuller tests on a
// time-series vector
// ------------------------------------------------------------
// REFERENCES: Said and Dickey (1984), "Testing for Unit Roots
// in Autoregressive Moving Average Models of Unknown Order", 
// Biometrika, Volume 71, pp. 599-607.
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * grocer_p = order of time polynomial in the null-hypothesis
//   . grocer_p = -1, no deterministic part
//   . grocer_p =  0, for constant term
//   . grocer_p =  1, for constant plus time-trend
//   . grocer_p >  1, for higher order polynomial
// * grocer_l = # of lags of the ADF test
// * grocer_np = 'noprint'if the user doesn't want to print
//     the results of the regression
// ------------------------------------------------------------
// OUTPUT:
// resadf = a results tlist with
//   . resadf('meth')  = 'adf'
//   . resadf('y')     = y data vector of the auxiliary
//     regression
//   . resadf('x')     = x data matrix of the auxiliary
//     regression
//   . resadf('nobs')  = # observations
//   . resadf('nvar')  = # variables
//   . resadf('beta')  = bhat
//   . resadf('yhat')  = yhat
//   . resadf('resid') = residuals of the auxiliary regression
//   . resadf('vcovar') = estimated variance-covariance matrix of
//     beta
//   . resadf('sige')  = estimated variance of the residuals
//   . resadf('sigu')  = sum of squared residuals
//   . resadf('ser')  = standard error of the regression
//   . resadf('tstat') = t-stats
//   . resadf('pvalue') = pvalue of the betas
//   . resadf('dw')    = Durbin-Watson Statistic
//   . resadf('condindex') = multicolinearity cond index
//   . resadf('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . resadf('rsqr')  = rsquared
//   . resadf('rbar')  = rbar-squared
//   . resadf('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . resadf('pvaluef') = its significance level
//   . resadf('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . resadf('namey') = name of the y variable of the auxiliary
//     regression
//   . resadf('namex') = name of the x variables of the auxiliary
//     regression
//   . resadf('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
//   . resadf('like') = log-likelihood of the regression
//   . resadf('1% level') = 1% critical level
//   . resadf('5% level') = 5% critical level
//   . resadf('10% level') = 10% critical level
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted to scilab from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
// and Modeled after a similar Gauss routine by
// Sam Ouliaris, in a package called COINT
 
grocer_prt=%t

// error checking on inputs

// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[x,grocer_namey,prests]=exploy(grocer_namey)

grocer_namexos=[]
if grocer_p < (-1) then
   error('p less than -1 in adf')
end
if grocer_p >= 0 then 
   prescte = %t
   grocer_namexos=[grocer_namexos ; 'cte']
end
if grocer_p == 1 then
   grocer_namexos=[grocer_namexos ; 't']
end
select typeof(x)
case 'constant' then
   nobs=size(x,1)
case 'ts' then
   nobs=size(ts2vec(x),1)
end

[nargout,nargin] = argn(0)
if nargin == 4 then
   if grocer_np == 'noprint' then
      prt=%f
   else
      error('argument 4 in adf should be ''noprint''')
   end
end

if (nobs-2*grocer_l+1)<1 then
   error('nlags too large in adf, negative dof');
end


if prests then
   dat=x('dates')
   deldates=dat(size(dat,1))-dat(1)+1
   t=reshape([1:deldates],num2date(dat(1),x('freq')))
   ch='resadf=ols(delts(x),lagts(x)'
   for i=1:grocer_l
      ch=ch+',delts(lagts('+string(i)+',x))'
   end
   if grocer_namexos ~= [] then
      ch=ch+','+joinstr('''',grocer_namexos,''',')+''',''noprint'')'
   end
   execstr(ch)
else
   if size(x,2) > 1 then
      error('adf cannot handle a matrix -- only vectors')
   end
 

   dx=tdiff(x,1)
   dxk=dx 
   depend= trimr(dx,grocer_l+1,0);
   z = trimr(lag(x,1),grocer_l+1,0)
   for k=1:grocer_l
      dxk=lag(dxk,1)
      z = [z trimr(dxk,grocer_l+1,0)];
   end
 
   if grocer_p>(-1) then
      z = [z,ptrend(grocer_p,size(z,1))];
   end

   resadf = ols2(depend,z)
   resadf(1)($+1) = 'prests'
   resadf('prests') = %f
   resadf(1)($+1) = 'namex'
   resadf(1)($+1) = 'namey'

end 
 
for i=grocer_l:-1:1
   grocer_namexos=['del('+grocer_namey+'(t-'+string(i)+'))' ; grocer_namexos ]
end 
grocer_namexos=[grocer_namey+'(t-1)' ; grocer_namexos]

// change the method name
resadf('meth') = 'ADF'

resadf('namex')=grocer_namexos
resadf('namey')='del('+grocer_namey'+')'

crit=ztcrit(resadf('nobs'),grocer_p)
resadf(1)($+1) = '1% level'
resadf(1)($+1) = '5% level'
resadf(1)($+1) = '10% level'
resadf('1% level') = crit(1)
resadf('5% level') = crit(2)
resadf('10% level') = crit(3)

if grocer_prt then
   prtunitr(resadf,%io(2))
end

 
