function [rescadf]=cadf(grocer_p,grocer_l,grocer_namey,varargin)

// PURPOSE: compute augmented Dickey-Fuller statistic for 
// residuals from a cointegrating regression, allowing for 
// deterministic polynomial trends
// ------------------------------------------------------------
// References: Said and Dickey (1984) 'Testing for Unit Roots 
// in Autoregressive Moving Average Models of Unknown Order', 
// Biometrika, Volume 71, pp. 599-607.
// ------------------------------------------------------------
// INPUT:
// * p = order of time polynomial in the null-hypothesis
//   . p = -1, no deterministic part
//   . p =  0, for constant term
//   . p =  1, for constant plus time-trend
//   . p >  1, for higher order polynomial
// * l = # of lagged changes of the residuals to include in 
//   regression
// * namey = a time series, a real (nx1) vector or a string 
// equal to the name of a time series or a (nx1) real vector
// between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
// ------------------------------------------------------------
// OUTPUT: 
// rcadf = a tlist with
//   . all of the arguments of the second stage regression
//   and
//   . rcadf('cointrel') = tlist with all the arguments of the
//   first stage regression
// (see ols() for a description of all these arguments)
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
// Modeled after a similar Gauss routine by
// Sam Ouliaris, in a package called COINT

nargin=length(varargin)

// error checking
if grocer_p<(-1) then
  error('p cannot be < -1 in cadf');
end

// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[grocer_y,grocer_namey,grocer_prests,grocer_boundsvarb]=...
exploy(grocer_namey)

// separate from the list of variable arguments the list of
// exogenous variables and 'noprint' if the user has given this
// argument
grocer_lx=varargin
[grocer_lx,grocer_lnp]=explolist(grocer_lx,...
list('grocer_lx(i)==''noprint'''))

// explode the list of the arguments into the corresponding
// variable, its name, and, if necessary updates the bounds
[grocer_x,grocer_namexos,grocer_boundsvarb]=...
explox(grocer_lx,'exogenous',grocer_boundsvarb)

// if necessary, define y on the admissible bounds
if typeof(grocer_y) == 'ts' then
   grocer_y=ts2vec(grocer_y,grocer_boundsvarb)
else
   if typeof(grocer_y) ~= 'constant' then
      error('exogenous variable '+namey+' is neither a ...
            time series nor a constant vector')
   end
end

if grocer_p >= 0 then
   n=size(grocer_y,1)
   grocer_x=[grocer_x ptrend(grocer_p,n)]
   grocer_namexos=[grocer_namexos ; 'cte']
   if grocer_p > 0 then
      grocer_namexos=[grocer_namexos ; 't' ]
   end
   if grocer_p > 1 then
      grocer_namexos=[grocer_namexos ; 't^'+string([2:grocer_p])]
   end
end

// provides the results from the regression of the vector y
// on the vector x
rols=ols2(grocer_y,grocer_x)

// saves the names, the bounds if the regression involves ts
rols(1)($+1) = 'prests'
rols(1)($+1) = 'namex'
rols(1)($+1) = 'namey'
rols('prests')=grocer_prests
rols('namex')=grocer_namexos
rols('namey')=grocer_namey
if grocer_prests then
   rols(1)($+1) = 'bounds'
   rols('bounds')=grocer_boundsvarb
end

if grocer_prests then
   bo=rols('bounds')
   rols_resid=res2ts(rols,'resid')
   grocer_boundsvar=lagbounds(grocer_boundsvarb,grocer_l+1)
   ch='ols(delts(rols_resid),lagts(1,rols_resid)'
   for i=1:grocer_l
      ch=ch+',delts(lagts('+string(i)+',rols_resid))'
   end
   ch=ch+',''noprint'')'
   rescadf=evstr(ch)
else
   resid=rols('resid')
   dep = tdiff(resid,1);
   dep = trimr(dep,1,0);
   k = 0;
   z = trimr(lag(resid,1),1,0);
   k = k+1;
   while k<=grocer_l then
      z = [z,lag(dep,k)];
      k = k+1;
   end
   z = trimr(z,grocer_l,0);
 
   dep = trimr(dep,grocer_l,0);
   [nobs,nvar]=size(dep)
 
   rescadf=ols2(dep,z)
end

grocer_namexos=['res(-1)']
for i=1:grocer_l
   grocer_namexos=[grocer_namexos 'del(res(-'+string(i)+"))"]
end

rescadf('meth')='cadf'
rescadf(1)($+1) = 'prests'
rescadf(1)($+1) = 'namex'
rescadf(1)($+1) = 'namey'
rescadf('prests')=grocer_prests
rescadf('namex')=grocer_namexos
rescadf('namey')='del(res)'
rescadf(1)($+1) = 'cointrel'
rescadf('cointrel') = rols
rescadf(1)($+1) = 'trend'
rescadf('trend') = grocer_p

crit=rztcrit(rescadf('nobs'),size(rols('x'),2)-grocer_p-1,grocer_p)

rescadf(1)($+1) = '1% level'
rescadf('1% level') = crit(1)
rescadf(1)($+1) = '5% level'
rescadf('5% level') = crit(2)
rescadf(1)($+1) = '10% level'
rescadf('10% level') = crit(3)

if length(grocer_lnp) == 0 then
   prtunitr(rescadf) 
end
