function [result]=johansen(grocer_p,grocer_k,varargin)

// PURPOSE: perform Johansen cointegration tests
// ------------------------------------------------------------
// * References: Johansen (1988), 'Statistical Analysis of 
// Co-integration vectors', Journal of Economic Dynamics and
// Control, 12, pp. 231-254.
// * MacKinnon, Haug, Michelis (1996) 'Numerical distribution 
// functions of likelihood ratio tests for cointegration', 
// Queen's University Institute for Economic Research
// Discussion paper.
// * (see also: MacKinnon's JBES 1994 article)
// ------------------------------------------------------------
// INPUT:
// * p = order of time polynomial in the null-hypothesis
//                 p = -1, no deterministic part
//                 p =  0, for constant term
//                 p =  1, for constant plus time-trend
//                 p >  1, for higher order polynomial
// * k = number of lagged difference terms used when
//                 computing the estimator
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
// ------------------------------------------------------------
// OUTPUT:
// result =a results tlist:
//   - result('meth')  = 'johansen'
//   - result('namex')  = the name of the variables (m x 1)
//   - result('x')  = matrix of values for the variables
//                    (m x 1)
//   - result('nobs')  = # of observations
//   - result('nvar')  = # of variables
//   - result('nlags')  = k
//   - result('trend')  = p
//   - result('eig')  = eigenvalues  (m x 1)
//   - result('evec') = eigenvectors (m x m), where first
//                      r columns are normalized coint vectors
//   - result('lr1')  = likelihood ratio trace statistic for 
//                       r=0 to m-1 (m x 1) vector
//   - result('lr2')  = maximum eigenvalue statistic for r=0 
//                       to m-1 (m x 1) vector
//   - result('cvt')  = critical values for trace statistic
//                        (m x 3) vector [90% 95% 99%]
//   - result('cvm')  = critical values for max eigen value
//                       statistic (m x 3) vector [90% 95% 99%]                            
//   - result('ind')  = index of co-integrating variables 
//                      ordered by size of the eigenvalues from 
//                      large to small                                       
// ------------------------------------------------------------
// NOTE: c_sja(), c_sjt() provide critical values generated
// using a method of MacKinnon (1994, 1996).
// Critical values are available for n<=12 and -1 <= p <= 1,
// zeros are returned for other cases.
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the trace and max eignevalues statistics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// ************************************************************
// NOTE: Adina Enache provided some bug fixes and corrections
// that she noted below in comments. 4/10/2000
// ************************************************************
 
// separates from the list of variable arguments the list of 
// exogenous variables and 'noprint' if the user has given this
// argument

grocer_ly=varargin
[grocer_ly,grocer_lnp]=explolist(grocer_ly,...
list('grocer_ly(i)==''noprint'''))

// explodes the list of the arguments into the corresponding  
// variable, its name, and, if necessary updates the bounds
[grocer_y,grocer_namey,grocer_boundsvarb,grocer_prests]=...
explox(grocer_ly,'endogenous')

[nobs,m] = size(grocer_y);

if grocer_p>(-1) then
  f = 0;
else
  f = grocer_p;
end
 
x = detrend(grocer_y,grocer_p);
dx = tdiff(x,1);
dx = trimr(dx,1,0);
r0t = detrend(trimr(dx,grocer_k,0),f);
rkt = detrend(trimr(lag(x,grocer_k),grocer_k+1,0),f);

if grocer_k ~= 0 then
   z = mlag(dx,grocer_k);
   z = detrend(trimr(z,grocer_k,0),f);
   r0t = r0t-z*(z\r0t);
   rkt = rkt-z*(z\rkt);
else
   error('in johansen there should be at least one lag')
end 

 
skk = rkt'*rkt/size(rkt,1);
sk0 = rkt'*r0t/size(rkt,1);
 
s00 = r0t'*r0t/size(r0t,1);
sig = sk0*inv(s00)*sk0';
tmp = inv(skk);
[au,du]=bdiag(tmp*sig,1/%eps)
orig = tmp*sig;

// Normalize the eigen vectors such that (du'skk*du) = I 
dt = du*inv(chol(du'*skk*du));
temp = inv(chol(du'*skk*du));
 
[a,aind]=gsort(diag(au),'r','d')
d = dt(:,aind);
 
// Compute the trace and max eigenvalue statistics 
lr1 = zeros(m,1);
lr2 = zeros(m,1);
cvm = zeros(m,3);
cvt = zeros(m,3);
iota = ones(m,1);
[t,junk] = size(rkt);
for i = 1:m
  tmp = trimr(log(iota-a),i-1,0);
  lr1(i,1) = -t*sum(tmp);
  lr2(i,1) = -t*log(1-a(i,1));
  cvm(i,:) = c_sja(m-i+1,grocer_p);
  cvt(i,:) = c_sjt(m-i+1,grocer_p);  
  aind(i) = i
end
 
// set up results tlist
result = tlist(['results';'meth';'namey';'y';'nobs';'nvar';...
'nlags';'trend';'eig';'evec';'lr1';'lr2';'cvt';'cvm';'ind']...
,'johansen',grocer_namey,grocer_y,nobs,m,grocer_k,grocer_p,...
a,d,lr1,lr2,cvt,cvm,aind)
result(1)($+1) = 'prests'
result('prests')=grocer_prests
if grocer_prests then
   result(1)($+1) = 'bounds'
   result('bounds')=grocer_boundsvarb
end


if length(grocer_lnp) == 0 then
   prtjohan(result,%io(2))
end
