function [resphil]=phil_perr(grocer_namey,grocer_p,grocer_l,grocer_np)

// PURPOSE: compute Phillips-Perron test of a unit root 
// hypothesis based on a Dickey-Fuller/Augmented Dickey-Fuller
// regression
// ------------------------------------------------------------
// REFERENCES:  
// Phillips, Peter (1987), "Time Series Regression with a Unit  
// Root", Econometrica, vol. 55, no. 2 (March), pp. 277-301
// Phillips, Peter & Pierre Perron (1988), "Testing for a Unit
// Root in Time Series Regression", Biometrika, vol. 75, no. 2
// (June), pp. 335-346
// ------------------------------------------------------------
// INPUT: 
// * grocer_namey = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * p = 0 if  
// * l = size of the Newey-West window
// ------------------------------------------------------------
// OUTPUT: 
// resphil = a tlist with
//   . resphil('meth')  = 'phillips-perron'
//   . resphil('y')     = y data vector
//   . resphil('x')     = x data matrix
//   . resphil('nobs')  = nobs
//   . resphil('nvar')  = nvars
//   . resphil('beta')  = bhat
//   . resphil('yhat')  = yhat
//   . resphil('resid') = residuals
//   . resphil('vcovar') = estimated variance-covariance matrix
//     of beta
//   . resphil('sige')  = estimated variance of the residuals
//   . resphil('sige')  = estimated variance of the residuals
//   . resphil('ser')  = standard error of the regression
//   . resphil('tstat') = t-stats
//   . resphil('pvalue') = pvalue of the betas
//   . resphil('dw')    = Durbin-Watson Statistic
//   . resphil('condindex') = multicolinearity cond index
//   . resphil('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . resphil('rsqr')  = rsquared
//   . resphil('rbar')  = rbar-squared
//   . resphil('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . resphil('pvaluef') = its significance level
//   . resphil('rho') = Phillips-perron statistic, 
//                     autocorrelation/heteroskedasticity 
//                     corrected value for the unit-root  
//                     coefficient based on a Dickey-Fuller
//                     regression
//   . resphil('tstat') = Phillips-perron statistic, 
//                     autocorrelation/heteroskedasticity 
//                     corrected t-ratio for the unit-root 
//                     coefficient based on a Dickey-Fuller
//                     regression
//   . resphil('crit_al')  = (3 x 1) vector of critical values
//                        [1% 5% 10%] quintiles for rho   
//   . resphil('crit_t')  = (3 x 1) vector of critical values
//                        [1% 5% 10%] quintiles for tstat  
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
 
grocer_prt=%t
[nargout,nargin]=argn(0)

// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[y,namey,prests,boundsvarb]=exploy(grocer_namey)

// if necessary, define y on the admissible bounds
if typeof(y) == 'ts' then
   y=ts2vec(y,boundsvarb)
else
   if typeof(y) ~= 'constant' then
      error('variable '+namey+' is neither a ...
            time series nor a constant vector')
   end
end

nobs = size(y,1)
x=lag(y,1)
x=[x(2:nobs,1) ptrend(grocer_p,nobs-1)] 
y=y(2:nobs)
nvar=grocer_p+2

if prests & exists('grocer_boundsvar') then
   if size(grocer_boundsvar,1) ~= 2 then
      error('bounds are discontinous in kpss')
   end
end

nobs=size(y,1)
[nargout,nargin] = argn(0)
select nargin
case 4 then
   if grocer_np == 'noprint' then
      grocer_prt=%f
   else
      error('argument 4 in adf should be ''noprint''')
   end
case 3 then
   if typeof(grocer_l) ==  'string' then
      if grocer_l == 'noprint' then
        grocer_prt=%f
        grocer_l=floor(5*nobs^0.25)
      else       
         error('argument 3 in adf cannot be: '+grocer_l)
      end
   elseif typeof(grocer_l) ~= 'constant' then 
      error('argument 3 in adf cannot have the type: '+typeof(grocer_l))
   end
case 2 then
   grocer_l=floor(5*nobs^0.25)
else
   error('invalid number of arguments')
end

nobs=nobs-1
if grocer_p==1 then
   x(:,3)=x(:,3)-nobs/2
end

resphil=ols2(y,x)
bet=resphil('beta')
ly=x(:,1)
resid=resphil('resid')
sigma2u=resid'*resid/nobs
sigma2=newey_west(resid,grocer_l)
h1=resphil('vcovar')(1,1)/resphil('sige')

rho=nobs*(bet(1,1)-1)-0.5*(sigma2-sigma2u)*h1*(nobs)^2
tstat=sqrt(sigma2u/sigma2)*(bet(1,1)-1)/sqrt(resphil('vcovar')(1,1))...
-0.5*(sigma2-sigma2u)/sqrt(sigma2)*nobs*sqrt(h1)

resphil('meth') = 'phillips-perron'
resphil(1)($+1) = 'rho'
resphil('rho') = rho
resphil(1)($+1) = 'tstat'
resphil('tstat') = tstat
resphil(1)($+1) = 'p'
resphil('p') = grocer_p

resphil(1)($+1) = 'prests'
resphil(1)($+1) = 'namey'
resphil('prests')=prests
resphil('namey')=namey


if prests then
   resphil(1)($+1) = 'bounds'
   resphil('bounds')=boundsvarb
end

[crit_al,crit_t]=fuller_tab(grocer_p,nobs)

resphil(1)($+1) = 'cv_rho_1%'
resphil('cv_rho_1%') = crit_al(1)
resphil(1)($+1) = 'cv_rho_5%'
resphil('cv_rho_5%') = crit_al(2)
resphil(1)($+1) = 'cv_rho_10%'
resphil('cv_rho_10%') = crit_al(3)

resphil(1)($+1) = 'cv_tstat_1%'
resphil('cv_tstat_1%') = crit_t(1)
resphil(1)($+1) = 'cv_tstat_5%'
resphil('cv_tstat_5%') = crit_t(2)
resphil(1)($+1) = 'cv_tstat_10%'
resphil('cv_tstat_10%') = crit_t(3)
 
if grocer_prt then
   prtunitr(resphil)
end

