function [resulsp]=schmiphi(grocer_namey,grocer_p,grocer_l,grocer_np)

// PURPOSE: computes Schmidt-Phillips test
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * grocer_p=order of time polynomial in the null-hypothesis
//      grocer_p =  0, for constant term
//      grocer_p =  1, for constant plus time-trend 
// * grocer_l (optional) = truncation lag of the Newey-West
//   windows
//   default : l= floor(5*nobs^0.25)
// * grocer_np (optional) = 'noprint' if the user doesn't want
// to print the results of the regression
// ------------------------------------------------------------
// OUPTUT:
// resulsp = results tlist with:
// - resulsp('meth') = 'schmiphi'
// - resulsp('namey') = name of the tested variable
// - resulsp('y') = (nobsx1) vector of endogenous variables
// - resulsp('nobs') = # of observations
// - resulsp('t') = order of the polynomial trend
// - resulsp('lag(NW)') = # of lags of the Newey-West window
// - resulsp('phi') = value of the phi test
// - resulsp('rho') = rho statistics
// - resulsp('tstat') = tstat statisctics
// - resulsp('v_rho_1%') = critical value of the rho-test at
//                        the 1% level
// - resulsp('v_rho_5%') = critical value of the rho-test at
//                        the 5% level
// - resulsp('v_rho_10%') = critical value of the rho-test at
//                        the 10% level
// - resulsp('v_tstat_1%') = critical value of the tstat-test at
//                        the 1% level
// - resulsp('v_tstat_5%') = critical value of the tstat-test at
//                        the 5% level
// - resulsp('v_tstat_10%') = critical value of the tstat-test at the
//                       10% level
// - resulsp('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - resulsp('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
// ------------------------------------------------------------
// REFERENCE:
// Schmidt, P. and Phillips P.C. (1992) : "LM TEST FOR A UNIT 
// ROOT IN THE PRESENCE OF DETERMINISTIC TRENDS", Oxford 
// Bulletin of economics and Statistics, pp. 257-287. 
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002

grocer_prt=%t

// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[y,namey,prests,boundsvarb]=exploy(grocer_namey)

// if necessary, define y on the admissible bounds
if typeof(y) == 'ts' then
   y=ts2vec(y,boundsvarb)
else
   if typeof(y) ~= 'constant' then
      error('variable '+namey+' is neither a ...
            time series nor a constant vector')
   end
end
     
if prests & exists('grocer_boundsvar') then
   if size(grocer_boundsvar,1) ~= 2 then
      error('bounds are discontinous in kpss')
   end
end

nobs=size(y,1)
[nargout,nargin] = argn(0)
select nargin
case 4 then
   if grocer_np == 'noprint' then
      prt=%f
   else
      error('argument 4 in adf should be ''noprint''')
   end
case 3 then
   if typeof(grocer_l) ==  'string' then
      if grocer_l == 'noprint' then
        grocer_prt=%f
        grocer_l=floor(5*nobs^0.25)
      else       
         error('argument 3 in adf cannot be: '+grocer_l)
      end
   elseif typeof(grocer_l) ~= 'constant' then 
      error('argument 3 in adf cannot have the type: '+typeof(grocer_l))
   end
case 2 then
   grocer_l=floor(5*nobs^0.25)
else
   error('invalid number of arguments')
end

ksi= (y(nobs)-y(1))/(nobs-1)
psi=y(1)-ksi
sbar=y(1:nobs-1)-psi-ksi*[0:nobs-2]'
dy=y(2:nobs)-y(1:nobs-1)

r=ols1(dy,[sbar ptrend(grocer_p,nobs-1)])
phi=r('beta')(1)
resid=r('resid')

// apply the newey-west correction to the statistics
correc=r('sigu')/nobs/newey_west(r('resid'),grocer_l)
rho=nobs*phi/correc
tstat=r('tstat')(1)/sqrt(correc)

[critical1,critical2]=schmiphi_tab(grocer_p,nobs)

resulsp=tlist(['results';'meth';'namey';'y';'nobs';'p';'lag(NW)';...
'phi';'rho';'tstat';'cv_rho_1%';'cv_rho_5%';'cv_rho_10%';...
'cv_tstat_1%';'cv_tstat_5%';'cv_tstat_10%';'prests'],'Schmidt-Phillips',...
namey,y,nobs,grocer_p,grocer_l,phi,rho,tstat,...
critical1(1),critical1(3),critical1(4),critical2(1),...
critical2(3),critical2(4),prests)

if prests then
   resulsp(1)($+1) = 'bounds'
   resulsp('bounds') = boundsvarb
end

if grocer_prt then
   prtunitr(resulsp,%io(2))
end
