function [results]=dfbeta(grocer_namey,varargin)

// PURPOSE: computes BKW (influential observation diagnostics)
//          dfbetas, dffits, hat-matrix, studentized residuals
// ------------------------------------------------------------
// REFERENCE: Belsley, Kuh, Welsch, 1980 Regression Diagnostics
// ------------------------------------------------------------
// INPUT: 
// * grocer_namey = either an ols results tlist or a a time
//   series, a real (nx1) vector or a string equal to the name
//   of a time series or a (nx1) real vector between quotes
// * varargin = if arguments which can be:
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
//   or, and only if grocer_namey is not an ols results tlist:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
// ------------------------------------------------------------
// OUTPUT: 
// results = a results tlist with
// - results('meth')   = 'dfbeta'
// - results('nobs')   = # of observations
// - results('nvar')   = # of variables in x-matrix
// - results('dfbeta') = df betas
// - results('dffits') = df fits
// - results('hatdi')  = hat-matrix diagonals
// - results('stud')   = studentized residuals
// - result('namex') = name of the x variables
// - result('namey') = name of the y variable
// - result('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// ------------------------------------------------------------
// Copyright Eric Dubois 2002 
// adapted by from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatia-econometrics.com
 
grocer_prt=%t
if typeof(grocer_namey) == 'results' then
   if grocer_y('meth') ~= ols then
      error('tlist argument should be an ols result')
   end  
   grocer_namexos=grocer_y('namex')
   grocer_namey=grocer_y('namey')
   grocer_prests=grocer_y('prests')
   if grocer_prests then
      grocer_boundsvarb=grocer_y('bounds')
   end
   grocer_x=grocer_y('x')
   [nobs,nvar] = size(grocer_x);
   bols=grocer_y('beta')
   e=grocer_y('resid')
   sige=grocer_y('sige')
   sxx=grocer_y('vcovar')/sige
   nargin=length(varargin)
   if nargin ~= 0 then
      if nargin ~= 1 then
         error ('too many arguments after the results tlist...
         argument')
      elseif varargin(1) == 'noprint' then
         grocer_prt=%t
      else
         error ('invalid argument')
      end
   end
else
   // explode namey into the corresponding variable, its name, if
   // it's a ts and if necessary the admissible bounds
   [grocer_y,grocer_namey,grocer_prests,grocer_boundsvarb]=...
   exploy(grocer_namey)

   // separate from the list of variable arguments the list of
   // exogenous variables and 'noprint' if the user has given this
   // argument
   grocer_lx=varargin
   [grocer_lx,grocer_lnp]=explolist(grocer_lx,...
   list('grocer_lx(i)==''noprint'''))
   if length(grocer_lnp) ~= 0 then
      grocer_prt=%f
   end

   // explode the list of the arguments into the corresponding
   // variable, its name, and, if necessary updates the bounds
   [grocer_x,grocer_namexos,grocer_boundsvarb]=...
   explox(grocer_lx,'exogenous',grocer_boundsvarb)

   // if necessary, define y on the admissible bounds
   if typeof(grocer_y) == 'ts' then
      grocer_y=ts2vec(grocer_y,grocer_boundsvarb)
   else
      if typeof(grocer_y) ~= 'constant' then
         error('exogenous variable '+namey+' is neither a ...
            time series nor a constant vector')
      end
   end
   sxx=invxpx(grocer_x)
   bols=ols0(y,x)
   e = y-x*bols;
   [nobs,nvar] = size(grocer_x);
   sige = e'*e/(nobs-nvar);
end

e2 = e .* e;

// perform QR decomposition
[q,r] = qr(grocer_x);
// pull out first nvar columns of q'
qt = q(1:nvar,:);
 
// get OLS estimates and related stuff
 
// find hat-matrix
h = zeros(nobs,1);
for i = 1:nobs
  h(i,1) = qt(:,i)'*qt(:,i);
end
 
omh = ones(nobs,1)-h;
homh = sqrt(h ./ omh);
 
d1 = (nobs-nvar)/(nobs-nvar-1)*sige*ones(nobs,1);
d2 = e2 ./ ((nobs-nvar-1)*omh);
si = sqrt(d1-d2);
t1 = si .* sqrt(omh);
 
dffits = homh .* (e ./ t1);
 
g = e ./ omh;
g = diag(g);
 
dfb = r'\(qt*g);
dfbet = dfb(1:nvar,1:nobs)';
 

scale=ones(nobs,nvar) 
for i = 1:nvar
  scale(1:nobs,i) = si*sxx(i,i);
end
 
dfbetas = dfbet ./ scale;

results=tlist(['results';'meth';'nobs';'nvar';'dfbeta';...
'dffits';'hatdi';'stud';'namex';'namey';'prests'],...
'bkw',nobs,nvar,dfbet,dffits,h,e ./ (si .* sqrt(omh)),...
grocer_namexos,grocer_namey,grocer_prests)

if grocer_prests then
   results(1)($+1)='bounds'
   results('bounds')=grocer_boundsvarb
end
 
if grocer_prt then 
   plt_dfb(results)
   plt_dff(results)
end