function [R,mu]=cholp(H,S)

// PURPOSE: Computes the perturbed Cholesky factor R for the 
// symmetric matrix H, such that R'*R = H + mu*I. 
// ------------------------------------------------------------
// INPUT:
// * H = a (nxn) symmetric square matrix
// * S = either a vector with length equal to n 
// ------------------------------------------------------------
// OUTPUT:
// * R = a (nxn) square matrix
// * mu = a scalar
// ------------------------------------------------------------
// Copyright Jaime Terceiro, 1997/
// Eric Dubois 2003 for the Scilab translation
 
[nargout,nargin]=argn(0) 
[n,m] = size(H);
if n~=m then
  error('first input should be a square matrix')
end

scalin = 0;
if nargin ~= 2 then 
   S=0
elseif S~=0 then
   scalin = 1;
   if or(size(S)==1) then
      S = S(:);
   else
      S = diag(S);
   end
   if size(S,1)>=n then
      S = S(1:n,:);
      d = diag(1/S);
      H = d*H*d';
   else
      scalin=0
   end
end
 
sqrteps = sqrt(%eps);
maxdiag = max(diag(H));
mindiag = min(diag(H));
maxpos = max(0,maxdiag);
 
if mindiag<=sqrteps*maxpos then
  mu = 2*(maxpos-mindiag)*sqrteps-mindiag;
  maxdiag = maxdiag+mu;
else
  mu = 0;
end
 
maxoff = 0;
if n>1 then
  maxoff1 = zeros(n-1,1);
  for i = 1:n-1
    %v2 =max(abs(H(i,i+1:n)))
    maxoff1(i,1) = %v2(:);
  end
  maxoff = max(maxoff1);
end
 
if maxoff*(1+2*sqrteps)>maxdiag then
  mu = mu+maxoff-maxdiag+2*sqrteps*maxoff;
  maxdiag = maxoff*(1+2*sqrteps);
end
if maxdiag==0 then
  mu = 1;
  maxdiag = 1;
end
if mu>0 then
  H = H+mu*eye(n,n);
end
maxoffl = sqrt(max(maxdiag,maxoff/n));
 
[L,maxadd] = choldc(H,maxoffl);
 
if maxadd>0 then
  maxev = H(1,1);
  minev = H(1,1);
  for i = 1:n
    offrow =sum(abs(H(i,i+1:n)))+sum(abs(H(1:i-1,i)));
    maxev = max(maxev,H(i,i)+offrow);
    minev = min(minev,H(i,i)-offrow);
  end
  sdd = (maxev-minev)*sqrteps-minev;
  sdd = max(sdd,0);
  mu = min(maxadd,sdd);
  H = H+mu*eye(n,n);
  [L,maxadd] = choldc(H,0);
end
 
if scalin then
  d = diag(S);
  L = L*d;
end
 
R = L';
// Same return as Scilab's chol function
