function [f,z1,x0]=lffast(theta,theta2mat,z)

// PURPOSE: Fast evaluation of the exact likelihood function
// for any time-invariant SS model (in that case lffast outputs
// the same result that lfmod with vcond = 'idej'. It is valid
// for stationary and/or nonstationary models.
// ------------------------------------------------------------
// INPUT:
// * theta = (npx1) vector of starting values for the
//   parameters
// * theta2mat = a string vector of instructions that
//   transforms theta into the matrices FR, FS, AR, AS, V and G
// * z = matrix of endogenous and exogenous variables
// ------------------------------------------------------------
// OUTPUT:
// * f = function value
// * z1 = vector of measure innovations
// * x0 = state vector
// ------------------------------------------------------------
// Under these conditions, this function 
// Copyright (c) Jaime Terceiro, 1997
// Eric Dubois 2003 for the scilab tanslation
 

[nargout,nargin] = argn(0)
if nargin<3 then
  error('Incorrect number of arguments');
end
 
saveinn = 0;
if nargout>1 then
  saveinn = 1;
end
 
[Phi,Gam,E,H,D,C,Q,S,R] = theta2ss(theta,theta2mat);

n = size(z,1);
l = size(Phi,1);
r = max([size(Gam,2),size(D,2)]);
m = size(H,1)
 
if ~grocer_e4_innov(1) then
  // 
  [E,Q,U,iU] = sstoinn(Phi,E,H,C,Q,S,R);
  // 
else
  // 
  Q = C*Q*C';
  E = E*pinv(C);
   
  if grocer_e4_scaleb then
    U = cholp(Q,abs(Q));
  else
    U = cholp(Q);
  end
  iU = inv(U');
end
 
if r&(grocer_e4_econd==1|grocer_e4_econd==4) then
  // 
  if grocer_e4_econd==1 then
    u0 = mean(z(:,m+1:m+r,2));
  else
    u0 = z(1,m+1:m+r)';
  end
   
  [x0,Sigm,iSigm,nonstat] = djccl(Phi,E*Q*E',0,Gam*u0);
  // 
else
  [x0,Sigm,iSigm,nonstat] = djccl(Phi,E*Q*E',0);
end
 
WW = zeros(l,l);
WZ = zeros(l,1);
Phib = Phi-E*H;
Phibb0 = iU*H;
 
if saveinn then
  N = eye(l,l);
end
 
ff = 0;
 
z1 = zeros(m,n);

if r then
   for t = 1:n
   // main loop
      z1(:,t) = z(t,1:m)'-H*x0-D*z(t,m+1:m+r)';
      x0 = Phi*x0+Gam*z(t,m+1:m+r)'+E*z1(:,t);
      WW = WW+Phibb0'*Phibb0;
      WZ = WZ+Phibb0'*iU*z1(:,t);
      Phibb0 = Phibb0*Phib;
   // 
   end
else
   for t = 1:n
      z1(:,t) = z(t,:)'-H*x0;
      x0 = Phi*x0+E*z1(:,t);
      WW = WW+Phibb0'*Phibb0;
      WZ = WZ+Phibb0'*iU*z1(:,t);
      Phibb0 = Phibb0*Phib;
   end
end

ff = 2*n*sum(log(diag(U)))+sum((iU*z1).^2);
 
if or(isnan([WW,WZ]),1)|or(isinf([WW,WZ]),1) then
   error('Initial conditions are meaningless');
end
 
if ~nonstat then
  // stationary system
  // 
  [Ns,S,Ns] = svd(Sigm);
  k = matrix(find(diag(S)<grocer_e4_zeps),1,-1);
  if size(k,1)<l then
    if size(k,1) then
      //   
      N = Ns(:,1:max(k(1),2)-1);
      Sigm = S(1:max(k(1),2)-1,1:max(k(1),2)-1);
      WW = N'*WW*N;
      WZ = N'*WZ;
      // 
    end
    M = cholp(Sigm);
    T = cholp(eye(size(M,1),size(M,1))+M*WW*M');
    ff = ff+2*sum(log(diag(T)));
    if grocer_e4_econd==2 then
      T = cholp(WW);
      ff = ff-sum((T'\WZ).^2);
    else
      ff = ff-sum((T'\(M*WZ)).^2);
    end

  end
  // 
elseif nonstat==2 then
  // non stationary system
  // 
  T = cholp(WW);
  ff = ff+2*sum(log(diag(T)))-sum((T'\WZ).^2);
  // 
else
  // partially stationary
  // 
  S = svd(Sigm);
  T = cholp(iSigm+WW);
  ff = ff+2*sum(log(diag(T)))+sum(log(S(S>grocer_e4_zeps)));
  if grocer_e4_econd==2 then
    T = cholp(WW);
  end
  ff = ff-sum((T'\WZ).^2);
  // 
end
 
f = .5*(ff+n*m*log(2*%pi));
 
if saveinn then
  // 
   x0 = zeros(l,n+1);
   x0(:,1) = N*pinv(WW)*WZ;
   
   if r then
      for t = 1:n
         // 
         z1(:,t) = z(t,1:m)'-H*x0(:,t)-D*z(t,m+1:m+r)';
         x0(:,t+1) = Phi*x0(:,t)+Gam*z(t,m+1:m+r)'+E*z1(:,t);
      end
   else
      for t = 1:n
         // 
         z1(:,t) = z(t,:)'-H*x0(:,t);
         x0(:,t+1) = Phi*x0(:,t)+E*z1(:,t);
      end 
   end
   
   z1 = z1';
   x0 = x0';
  // 
end
