function [g,dht,gradt]=garch_grad2(parm,nar,nma,y,x)

// PURPOSE: gradient for garch model
// ------------------------------------------------------------
// INPUT:
// * parm = a vector of parmeters
//        parm(1) = beta 1
//        parm(2) = beta 2
//        .
//        .
//        .
//        parm(k) = beta k
//        parm(k+1) = a0
//        parm(k+2) = ar(1)
//        .
//        .
//        .
//        parm(k+nar+1) = ar(nar)
//        parm(k+nar+1) = ma(1)
//        .
//        .
//        .
//        parm(k+nar+nma+1) = ar(nma)
// * y = (n x 1) vector of the endogenous variable
// * x = (n x k) vector of the exogenous variables
// * nar = # of ar coefficient in the garch model
// * nma = # of ma coefficient in the garch model
// ------------------------------------------------------------
// OUTPUT: 
// * g = (k+nar+nma+1 x 1) -gradient at param
// * dht = (nobs x nar+nma+1) derivative of sigt w.r.t a0, ar,
//          ma
// * scores = (k+nar+nma+1 x 1) sub-gradient at each date               
// ------------------------------------------------------------
// REFERENCES: Fiorentini, Calzolari et Panattoni (1996):
// Journal of Applied Econometrics, col1 ,n4, pp 399-417
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
 
[n,k] = size(x);
b=parm(1:k)
// transform parameters
[parmt] = garch_trans(parm,k);
a0=parmt(k+1)
ar=parmt(k+2:k+1+nar)
ma=parmt(k+2+nar:k+1+nar+nma)

// compute the derivative of [ar;ma] with respect to...
// parm(1+nar+nma)
D=diag([ar ;ma])-[ar ; ma]*[ar ; ma]'

// compute residuals
res0 = y-x*b;
sig = res0'*res0/n
ser=sqrt(sig)
nmax=max(nar,nma)

res=[ser*ones(nmax,1) ; res0]
res2=res.*res
sigt = [sig*ones(nmax,1) ; zeros(n,1)]

// equation (11) in Fiorentini, Calzolari et Panattoni
c=[ones(nmax,1).*.(-2*(res0'*x)/n) ; zeros(n,k)]
d=zeros(n+nmax,1)
e=zeros(n+nmax,nar)
f=zeros(n+nmax,nma)

for i = 1+nmax:n+nmax
   sigt(i) = parmt(k+1:k+1+nar+nma)'*[1 ; sigt(i-nar:i-1) ;...
             res(i-nma:i-1).*res(i-nma:i-1)];
// equation (11) in Fiorentini, Calzolari et Panattoni
   c(i,:)=-2*(ma.*res(i-1:-1:i-nma)'*x(i-1:-1:i-nma,:))+...
          ar'*c(i-1:-1:i-nar,:)
// equation (7) in Fiorentini, Calzolari et Panattoni
   d(i)=1+ar'*d(i-1:-1:i-nar)'
   ei=sigt(i-1:-1:i-nar)+ar'*e(i-1:-1:i-nar,:)
   fi=res2(i-1:-1:i-nma)+ar'*f(i-1:-1:i-nar,:)
   g=D*[ei ; fi]
   e(i,:)=g(1:nar)'
   f(i,:)=g(nar+1:nar+nma)'
end
// adjust the derivative to take into account the 
// fact that a0 is squared
d=2*parm(k+1)*d
sigt=sigt(1+nmax:n+nmax,:)
base=ones(1,k+1+nar+nma).*.(((res0.^2)./sigt-1)./(2*sigt))

dht=[c';d';e';f']'
dht=dht(1+nmax:n+nmax,:)
gradt=base.*dht+[matmul(res0./sigt,x) zeros(n,1+nar+nma)]

g= -sum(gradt(1:n,:),'r')'
