function [rsur]=sur(varargin)

// PURPOSE: provide Zellner Seemingly Unrelated Regression 
// ------------------------------------------------------------
// INPUT:
// strings which can be:
// * 'noprint' if the user does not want to print the results
// * 'niter=x' where x is the max # iterations authorized 
//   (optional; default =100)   
// * 'crit=x' where x is the convergence criterion
//   (optional; default =1e-4)
// * 'coef=xx' where xx is a vector of coefficients names
// * equations written 'vary=coef1*varx1+...+coefi*varxi' 
//   where:
//   - coefi = the name of a coefficient   
//   - varxi = the name of a variable  
// ------------------------------------------------------------
// OUTPUT:
// rsur=a tlist with
// - rsur('meth') = 'sur'
// - rsur('nobs') = # of observations
// - rsur('neqs') = # of estimated equations
// - rsur('ncoef') = # of estimated coefficients
// - rsur('beta') = bhat
// - rsur('tstat') = t-stats
// - rsur('pvalue') = pvalue of the betas
// - rsur('sigma') = covariance matrix of the residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('sigu') = (1 x neqs) sum of squared residuals
// - rsur('dw') = (1 x neqs) Durbin-Watson
// - rsur('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - rsur('namecoef') = (ncoef x 1) mame of the coeffcients
// - rsur('namey') = name of endogenous variables
// - rsur('eqs') = list of the neqs equations
// - rsur('coefs') = list of the coefs names in each equation
// ------------------------------------------------------------
// NOTES:
// * some coefficients can be common to several equations
// * there can be spaces in the text of the equations
// * if you want to introduce a constant in your equation, you
//   can omit the '*varxi' in the text of the equation
// * the exogenous variable can be expressed anyway
// * the only constraint is that the model must be linear in
//   its coefficients
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002  

grocer_neqs=length(varargin)

// set defaults
grocer_itmax=100
grocer_crit=1e-4
grocer_defcoef=%f
grocer_prt=%t

// find if the user has given options, store them and keep the 
// equations in the varargin list
for grocer_i=grocer_neqs:-1:1
   grocer_eq=varargin(grocer_i)
   if grocer_eq == 'noprint' then
      grocer_prt=%f
      grocer_neqs=grocer_neqs-1
      varargin(grocer_i)=null()
   else
      grocer_indequal=strindex(varargin(grocer_i),'=')
      if size(grocer_indequal,2) ~= 1 then
         error('bad expression for',varargin(grocer_i))
      end
      if (part(varargin(grocer_i),1:6) == 'itmax='...
         | (part(varargin(grocer_i),1:5) == 'crit=')) then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
         grocer_neqs=grocer_neqs-1
      else
         if part(varargin(grocer_i),1:9) == 'namecoef=' then
            execstr('grocer_'+varargin(grocer_i))
            grocer_coef=str2vec(grocer_coef,',',';')
            grocer_ncoef=size(grocer_coef,1)
            grocer_defcoef=%t
            varargin(grocer_i)=null()
            grocer_neqs=grocer_neqs-1
         end
      end
   end
end

// if the user has not provided the names of the coefficients
// determine them
grocer_speccarb=['=' ; '+' ; '(' ; '-' ; '*']
grocer_speccara=['+' ; '-' ; '*' ; '/' ; ')']
if ~grocer_defcoef then
   grocer_namecoef=defaultcoef('a',grocer_speccarb,grocer_speccara,varargin(:))
end

// explode the equation into the x,y matrices, the name of the
// endogenous variables, the list of indexes of the coefficients
// in each equation and the bounds and the boolean indicating
// the presence of a ts
[x,y,grocer_boundsvarb,grocer_prests,listcoef,namey,ncoefeqs]=...
explosys(grocer_namecoef,grocer_speccara,grocer_speccarb,varargin(:))

convg=grocer_crit*2
// uses results from ols as starting values
beta0=ols0(y,x)
resid0=y-x*beta0
nbit=0
nobs=size(y,1)/grocer_neqs
ncoef=size(grocer_namecoef,1)

while (convg > grocer_crit & nbit < grocer_itmax) then
   nbit=nbit+1
   sigma=zeros(grocer_neqs,grocer_neqs)
   resd=resid0./(sqrt(nobs-ncoefeqs).*.ones(nobs,1))
   rest=matrix(resd,nobs,grocer_neqs)
   sigma=rest'*rest
   sqrtsigma=sigma^(-0.5).*.eye(nobs,nobs)
   sqrtsigmax=sqrtsigma*x
   [q,r]= qr(sqrtsigmax)
   q = q(:,1:size(r,2))
   r = r(1:size(r,2),:)
   ixpomegam1x=inv(r'*r)
   bhat=ixpomegam1x*sqrtsigmax'*sqrtsigma*y
   resid=y-x*bhat
   convg=sum(abs(bhat-beta0))
   beta0=bhat
   resid0=resid
end

resid=matrix(resid,nobs,grocer_neqs)
tstat=bhat./sqrt(diag(ixpomegam1x))
pvalue=zeros(ncoef,1)
ds=diag(sigma)
ser=sqrt(ds)
sigu=ds.*(nobs-ncoefeqs)
corm=var2cor(sigma)

for i=1:ncoef
   pvalue(i)=(1-cdfnor("PQ",abs(tstat(i)),0,1))*2
end

// durbin-watson
dw=zeros(1,grocer_neqs)
for i=1:grocer_neqs
   ediff = resid(2:nobs,i)-resid(1:nobs-1,i)
   dw(i) = ediff'*ediff/sigu(i)
end

rsur=tlist(['results';'meth';'nobs';'neqs';'ncoef';'beta';...
'tstat';'pvalue';'resid';'sigma';'corr';'sigu';'ser';'dw';...
'prests';'namecoef';'namey';'eqs';'coefs'],...
'sur',nobs,grocer_neqs,ncoefeqs,bhat,tstat,pvalue,...
resid,sigma,corm,sigu,ser,dw,grocer_prests,grocer_namecoef,...
namey,varargin,listcoef)

if grocer_prests then
   rsur(1)($+1) = 'bounds'
   rsur('bounds')=grocer_boundsvarb
end

if grocer_prt then 
   prtsys(rsur)
end
