function [rhwhite]=hwhite(grocer_namey0,varargin)

// PURPOSE: computes White's adjusted heteroscedastic
// consistent Least-squares Regression
// ------------------------------------------------------------
// References: H. White 1980, Econometrica Vol. 48 pp. 818-838. 
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
// ------------------------------------------------------------
// OUTPUT: 
// rhwhite = a tlist with
//   . rhwhite('meth')  = 'White''s heteroskedasticity
//                        consistent'
//   . rhwhite('y')     = y data vector
//   . rhwhite('x')     = x data matrix
//   . rhwhite('nobs')  = nobs
//   . rhwhite('nvar')  = nvars
//   . rhwhite('beta')  = bhat
//   . rhwhite('yhat')  = yhat
//   . rhwhite('resid') = residuals
//   . rhwhite('vcovar') = estimated variance-covariance matrix
//                         of beta
//   . rhwhite('sige')  = estimated variance of the residuals
//   . rhwhite('sige')  = estimated variance of the residuals
//   . rhwhite('ser')  = standard error of the regression
//   . rhwhite('tstat') = t-stats
//   . rhwhite('pvalue') = pvalue of the betas
//   . rhwhite('dw')    = Durbin-Watson Statistic
//   . rhwhite('prescte') = boolean indicating the presence or
//                     absence of a constant in the regression
//   . rhwhite('rsqr')  = rsquared
//   . rhwhite('rbar')  = rbar-squared
//   . rhwhite('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rhwhite('pvaluef') = its significance level
//   . rhwhite('prescte') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rhwhite('namey') = name of the y variable
//   . rhwhite('namex') = name of the x variables
//   . rhwhite('bounds') = if there is a timeseries in the 
//                     regression, the bounds of the regression
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 2004
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com
 
// separate from the list of variable arguments the list of
// exogenous variables and 'noprint' if the user has given this
// argument
grocer_lx=varargin
[grocer_lx,grocer_lnp]=explolist(grocer_lx,...
list('grocer_lx(i)==''noprint'''))

[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb]=explouniv(grocer_namey0,grocer_lx)

[nobs,nvar]=size(grocer_x)
if nvar >= nobs then
   error('too many exogenous variables')
end

xpxi=invxpx(grocer_x)
bet=xpxi*grocer_x'*grocer_y
yhat=grocer_x*bet
resid=grocer_y-yhat
sigu = resid'*resid;
sige=sigu/(nobs-nvar)
ser=sqrt(sige)
 
// perform White's correction
xuux = mcov(grocer_x,resid);
xpxia = xpxi*xuux*xpxi;
tmp = sqrt(diag(xpxia));
tstat=bet ./ tmp
 
ym = grocer_y-mean(grocer_y);
ediff = resid(2:nobs)-resid(1:nobs-1)
dw = ediff'*ediff/sigu
// durbin-watson

df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end

condindex=bkwols(grocer_x)

prescte=%f
i=1
while (i <= nvar) & ~prescte then
   // if all values are equal to the first one then, 
   // the variable is constant
   prescte=and(grocer_x(:,i) == grocer_x(1,i))
   i=i+1
end

rhwhite = tlist(['results';'meth';'y';'x';'nobs';'nvar';...
'beta';'yhat';'resid';'vcovar';'sige';'sigu';'ser';...
'tstat';'pvalue';'dw';'condindex';'prescte']...
,'White''s heteroskedasticity consistent',grocer_y,...
grocer_x,nobs,nvar,bet,yhat,resid,xpxia,sige,sigu,ser,tstat,...
pvalue,dw,condindex,prescte)


if prescte & nvar ~=1 then
// there is a constant and at least another exogenous variable:
// R and Rbar make sense
  rsqr1 = sigu;
  rsqr2 = ym'*ym;
  // r-squared
  rhwhite(1)($+1)='rsqr'
  rsqr=1-rsqr1/rsqr2
  rhwhite('rsqr') =rsqr 
  nobsm1=nobs-1
  nvarm1=nvar-1
  rhwhite(1)($+1)='rbar'
  rhwhite('rbar') = 1-rsqr1/df/rsqr2*nobsm1
  // rbar-squared
  f=rsqr/(1-rsqr)*df/(nvar-1)
  pvaluef=1-cdff("PQ",f,nvarm1,nobs-nvar)
  rhwhite(1)($+1)='f'
  rhwhite('f') = f
  rhwhite(1)($+1)='pvaluef'
  rhwhite('pvaluef') = pvaluef
end

rhwhite(1)($+1) = 'prests'
rhwhite(1)($+1) = 'namex'
rhwhite(1)($+1) = 'namey'
rhwhite('prests')=grocer_prests
rhwhite('namex')=grocer_namexos
rhwhite('namey')=grocer_namey
if grocer_prests then
   rhwhite(1)($+1) = 'bounds'
   rhwhite('bounds')=grocer_boundsvarb
end

if length(grocer_lnp) == 0 then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(rhwhite,foutput)
   prtuniv(rhwhite,%io(2))
end
