function [riv]=iv(grocer_namey0,varargin)

// PURPOSE: compute instrumental variables estimation for a
// single equation
// ------------------------------------------------------------
// INPUT:
// * grocer_namey = dependent variable vector (nobs x 1)
// * varargin is either
//   - the string 'exo=[var1;var2;...,vark]' where var1, var2,
//   ... , vark are exogenous variables which are therefore not
//   instrumented
//   - the string 'endo=[var1;var2;...,varl]' where var1, var2,
//   ... , varl are endogenous variables which are therefore
//   instrumented
//   - the string 'ivar=[var1;var2;...,varm]' where var1, var2,
//   ... , varm are the instruments
//  - the string 'noprint' if the user doesn't want to print 
//      the results of the regression
// ------------------------------------------------------------
// OUTPUT:
// results = a structure tlist with
// - riv('meth')  = 'iv'
// - riv('nobs')  = nobs
// - riv('nendog') = # of endogenous
// - riv('nexog')  = # of exogenous
// - riv('nvar')   = # of endogenous + # of exogenous
// - riv('y')      = y data vector
// - riv('beta')  = bhat estimates
// - riv('tstat') = t-statistics
// - riv('yhat')  = yhat predicted values
// - riv('resid') = residuals
// - riv('residtsls') = residuals calculated with the 
//   endogenous variables replaced by their regression from
//   first stage estimation
// - riv('sigu')  = e'*e
// - riv('sige')  = e'*e/(n-k)
// - riv('dw')    = Durbin-Watson Statistic
// - riv('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
// - riv('rsqr')  = rsquared
// - riv('rbar')  = rbar-squared
// - riv('f')    = F-stat for the nullity of coefficients
//    other than the constant
// - riv('pvaluef') = its significance level
// - riv('grsqr')  = generalized rsquared (that is which takes
//    into account the endogeneity of some explicative
//    variables)
// - riv('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - riv('namey') = name of the y variable
// - riv('namex') = name of the x variables
// - riv('nameinst') = name of the instruments
// - riv('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
// ------------------------------------------------------------
// Copyright Eric Dubois 2002

grocer_prt=%t
// explode namey into the corresponding variable, its name, if
// it's a ts and if necessary the admissible bounds
[grocer_y,grocer_namey,grocer_prests,grocer_boundsvarb]=...
exploy(grocer_namey0)

gorcer_nargin=length(varargin)

for grocer_i=1:gorcer_nargin

   grocer_st=varargin(grocer_i)

   if part(grocer_st,1:5) == 'endo=' then
      grocer_endo=str2vec(grocer_st)
   end 
  
   if part(grocer_st,1:4) == 'exo=' then
      grocer_exo=str2vec(grocer_st)
      grocer_nx=size(grocer_exo,1)
   end

   if part(grocer_st,1:5) == 'ivar=' then
      grocer_ivar=str2vec(grocer_st)
      grocer_niv=size(grocer_ivar,1)
   end

   if grocer_st == 'noprint' then
      grocer_prt=%f
   end
end

// explodes the list of the endogenous variables into the
// corresponding variables, their names, and, if necessary
// updates the bounds
[grocer_endo,grocer_nameendo,grocer_boundsvarb,grocer_prests]...
=explox(grocer_endo,grocer_boundsvarb)

// explodes the list of the exogenous variables into the
// corresponding variables, their names, and, if necessary
// updates the bounds
[grocer_exo,grocer_namexos,grocer_boundsvarb,grocer_prests]=...
explox(grocer_exo,grocer_boundsvarb)

// explodes the list of the instrumental variables into the
// corresponding variables, their names, and, if necessary
// updates the bounds
[grocer_ivar,grocer_nameivar,grocer_boundsvarb,grocer_prests]...
=explox(grocer_ivar,grocer_boundsvarb)

// if necessary, define y on the admissible bounds
if typeof(grocer_y) == 'ts' then
   grocer_y=ts2vec(grocer_namey0,grocer_boundsvarb)
else
   if typeof(grocer_y) ~= 'constant' then
      error('variable '+grocer_namey+' is neither a ...
            time series nor a constant vector')
   end
end

riv=iv1(grocer_y,grocer_endo,grocer_exo,grocer_ivar)

// saves the names, the bounds if the regression involves ts
riv(1)($+1) = 'prests'
riv(1)($+1) = 'namex'
riv(1)($+1) = 'nameendo'
riv(1)($+1) = 'namey'
riv(1)($+1) = 'nameinst'
riv('prests')=grocer_prests
riv('namex')=[grocer_nameendo ; grocer_namexos]
riv('nameendo')=grocer_nameendo
riv('namey')=grocer_namey
riv('nameinst')=grocer_nameivar
if grocer_prests then
   riv(1)($+1) = 'bounds'
   riv('bounds')=grocer_boundsvarb
end

if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(riv,foutput)
   prtuniv(riv,%io(2))
end
