function [rols]=ols2(y,x)

// PURPOSE: least-squares regression when variables 
// looks like in the textbook
// more efficient than ols (but less user-friendly)
// ------------------------------------------------------------
// INPUT:
// * y = dependent variable vector (nobs x 1)
// * x = independent variables matrix (nobs x nvar)
// ------------------------------------------------------------
// OUTPUT: 
// rols = a tlist with
//   . rols('meth')  = 'ols'
//   . rols('y')     = y data vector
//   . rols('x')     = x data matrix
//   . rols('nobs')  = nobs
//   . rols('nvar')  = nvars
//   . rols('beta')  = bhat
//   . rols('yhat')  = yhat
//   . rols('resid') = residuals
//   . rols('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rols('sige')  = estimated variance of the residuals
//   . rols('sige')  = estimated variance of the residuals
//   . rols('ser')  = standard error of the regression
//   . rols('tstat') = t-stats
//   . rols('pvalue') = pvalue of the betas
//   . rols('dw')    = Durbin-Watson Statistic
//   . rols('condindex') = multicolinearity cond index
//   . rols('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rols('rsqr')  = rsquared
//   . rols('rbar')  = rbar-squared
//   . rols('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rols('pvaluef') = its significance level
// ------------------------------------------------------------
// NOTES:
// * used by ols(), olsc()
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jlesage@spatial-econometrics.com

nobs2=size(y,1)
[nobs,nvar] = size(x);
if nobs~=nobs2 then
  error('x and y must have same # obs in ols2');
end
if nvar >= nobs then
   error('too many exogenous variables')
end

xpxi=invxpx(x)
bhat=xpxi*x'*y
yhat = x*bhat
resid = y-yhat
sigu =resid'*resid
sige= sigu/(nobs-nvar)
ser=sqrt(sige)
vcovar=sige*xpxi
tmp = diag(vcovar);
tstat = bhat ./ sqrt(tmp)

ym = y-mean(y);
ediff = resid(2:nobs)-resid(1:nobs-1)
dw = ediff'*ediff/sigu
// durbin-watson

df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end

condindex=bkwols(x)

prescte=%f
if nvar ~= 1 then
   i=1
   while (i <= nvar) & ~prescte then
      // if all values are equal to the first one then, 
      // the variable is constant
      prescte=and(x(:,i) == x(1,i))
      i=i+1
   end
end
llike=-0.5*nobs*(log(2*%pi)+log(sigu/nobs)+1)

rols = tlist(['results';'meth';'y';'x';'nobs';'nvar';'beta';'yhat';...
'resid';'vcovar';'sige';'sigu';'ser';'tstat';'pvalue';'dw';'condindex';...
'prescte';'llike']...
,'ols',y,x,nobs,nvar,bhat,yhat,resid,vcovar,sige,sigu,ser,tstat,...
pvalue,dw,condindex,prescte,llike)


if prescte then
// there is a constant and at least another exogenous variable:
// R and Rbar make sense
  rsqr1 = sigu;
  rsqr2 = ym'*ym;
  // r-squared
  rols(1)($+1)='rsqr'
  rsqr=1-rsqr1/rsqr2
  rols('rsqr') =rsqr 
  nobsm1=nobs-1
  nvarm1=nvar-1
  rols(1)($+1)='rbar'
  rols('rbar') = 1-rsqr1/df/rsqr2*nobsm1
  // rbar-squared
  if rsqr ~= 1 then
     f=rsqr/(1-rsqr)*df/(nvar-1)
  else
     warning('rsqr = 1: your exogenous variables are exactly colinear')
     f=%inf
  end
  pvaluef=1-cdff("PQ",f,nvarm1,nobs-nvar)
  rols(1)($+1)='f'
  rols('f') = f
  rols(1)($+1)='pvaluef'
  rols('pvaluef') = pvaluef
end
