function [result]=olsar1(grocer_namey0,varargin)

// PURPOSE: computes maximum likelihood ols regression for AR1
// errors
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a 
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
//---------------------------------------------------
// rolsar1 = a results tlist with
//   . rolsar1('meth')  = ' ar(1) maximum likelihood'
//   . rolsar1('y')     = y data vector
//   . rolsar1('x')     = x data matrix
//   . rolsar1('nobs')  = # observations
//   . rolsar1('nvar')  = # variables
//   . rolsar1('beta')  = bhat
//   . rolsar1('yhat')  = yhat
//   . rolsar1('resid') = residuals
//   . rolsar1('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rolsar1('sige')  = estimated variance of the residuals
//   . rolsar1('sigu')  = sum of squared residuals
//   . rolsar1('ser')  = standard error of the regression
//   . rolsar1('tstat') = t-stats
//   . rolsar1('pvalue') = pvalue of the betas
//   . rolsar1('dw')    = Durbin-Watson Statistic
//   . rolsar1('condindex') = multicolinearity cond index
//   . rolsar1('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rolsar1('rsqr')  = rsquared
//   . rolsar1('rbar')  = rbar-squared
//   . rolsar1('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rolsar1('pvaluef') = its significance level
//   . rolsar1('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rolsar1('namey') = name of the y variable
//   . rolsar1('namex') = name of the x variables
//   . rolsar1('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
//   . rolsar1('rho') = estimated first order autocorrelation of residuals
//   . rolsar1('trho') = its Student t
//   . rolsar1('like') = log-likelihood of the regression
// --------------------------------------------------

grocer_prt=%t

// separate from the list of variable arguments the list of
// exogenous variables and 'noprint' if the user has given this
// argument
grocer_lx=varargin
[grocer_lx,grocer_lnp]=explolist(grocer_lx,...
list('grocer_lx(i)==''noprint'''))
if length(grocer_lnp) ~= 0 then
   grocer_prt=%f
end

[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb]=explouniv(grocer_namey0,grocer_lx)
 
[nobs,nvar] = size(grocer_x);
[nobs2,junk] = size(grocer_y);
 
if nobs~=nobs2 then
  error('x and y must have same # obs in olsar1');
end
if nvar >= nobs then
   error('too many exogenous variables')
end
 
xtmp = lag(grocer_x,1);
ytmp = lag(grocer_y,1);
 
// truncate 1st observation to feed the lag
xlag = xtmp(2:nobs,:);
ylag = ytmp(2:nobs,1);
yt = grocer_y(2:nobs,1);
xt = grocer_x(2:nobs,:);
 
// use cochrane-orcutt estimates as initial values
 
reso = olsc(grocer_y,grocer_x,'noprint');
 
parm = zeros(nvar+1,1);
parm(1) = reso('rho');
// initial rho
parm(2:nvar+1) = reso('beta');
// initial bhat's
 
deff('[f,g,ind]=grocer_cost(parm,ind)',..
     ['f=ar1_like(parm,grocer_y,grocer_x)';...
     'g=ar1_grad(parm,grocer_y,grocer_x)'])

binf=[-1+%eps ; -%inf*ones(nvar,1)]

[like,bet] = optim(grocer_cost,'b',binf,-binf,parm);
llike = -like+nobs/2*(log(nobs)-log(2*%pi))
 
// after convergence produce a final set of estimates using rho-value
rho = bet(1,1);
ys = grocer_y-rho*lag(grocer_y);
xs = grocer_x-rho*lag(grocer_x);
ys(1,1) = sqrt(1-rho*rho)*grocer_y(1,1);
xs(1,:) = sqrt(1-rho*rho)*grocer_x(1,:);
 
// provides the results from the regression of the vector y
// on the vector x
result=ols2(ys,xs)

// saves the names, the bounds if the regression involves ts
result(1)($+1) = 'prests'
result(1)($+1) = 'namex'
result(1)($+1) = 'namey'
result('prests')=grocer_prests
result('namex')=grocer_namexos
result('namey')=grocer_namey
if grocer_prests then
   result(1)($+1) = 'bounds'
   result('bounds')=grocer_boundsvarb
end

result('meth')='ar(1) maximum likelihood'
// compute t-statistic for rho
varrho = (1-rho*rho)/(nobs-2);
result(1)($+1)='rho'
result('rho')=rho
result(1)($+1)='trho'
result('trho')=rho/sqrt(varrho)
result(1)($+1)='like'
result('like')=llike

if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
   prtuniv(result,%io(2))
   write(%io(2),' ')
   mat2print=['rho' string(rho); 't-stat' string(result('trho'))]
   printmat(mat2print,%io(2))
   printsep(%io(2))
end
