function [rolsc]=olsc(grocer_namey0,varargin)

// PURPOSE: compute Cochrane-Orcutt ols Regression for AR1
// errors
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a  
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nx1) vector 
//   . a string equal to the name of a time series or a (nx1) 
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
// ------------------------------------------------------------
// OUTPUT: 
// rolsc = a tlist with
//   . rolsc('meth')  = 'Cochrane-Orcutt'
//   . rolsc('y')     = y data vector
//   . rolsc('x')     = x data matrix
//   . rolsc('nobs')  = nobs
//   . rolsc('nvar')  = nvars
//   . rolsc('beta')  = bhat
//   . rolsc('yhat')  = yhat
//   . rolsc('resid') = residuals
//   . rolsc('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rolsc('sige')  = estimated variance of the residuals
//   . rolsc('sige')  = estimated variance of the residuals
//   . rolsc('ser')  = standard error of the regression
//   . rolsc('tstat') = t-stats
//   . rolsc('pvalue') = pvalue of the betas
//   . rolsc('dw')    = Durbin-Watson Statistic
//   . rolsc('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rolsc('rsqr')  = rsquared
//   . rolsc('rbar')  = rbar-squared
//   . rolsc('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rolsc('pvaluef') = its significance level
//   . rolsc('prescte') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rolsc('namey') = name of the y variable
//   . rolsc('namex') = name of the x variables
//   . rolsc('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
//   . rolsc('rho') = the autocorrelation coefficient of the
//     residuals
//   . rolsc('trho') = its T-stat
//   . rolsc('iterout') = a (niter x 3) matrix giving for each
//     iteration the estimated rho, the convergence criterion
//     and its number
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// Copyright: Eric Dubois 
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jpl@jpl.econ.utoledo.edu

// set default values
grocer_prt=%t
grocer_crit=sqrt(%eps)
grocer_maxit=100

// separate from the list of variable arguments the list of
// exogenous variables and options if any
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      if varargin(grocer_i) == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i)=null()
      elseif part(varargin(grocer_i),1:4) == 'crit' |...
         part(varargin(grocer_i),1:5) == 'maxit' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' then
      error('invalid type: '+typeof(varargin(grocer_i)))
   end
end

grocer_lx=varargin
[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb]=explouniv(grocer_namey0,grocer_lx)
 
// ----- setup parameters
converg = 1;
rho = 0;
iter = 1;
[nobs,nvar]=size(grocer_x)
[nobs2]=size(grocer_y,1)
if nobs~=nobs2 then
  error('x and y must have same # obs in ols2');
end
if nvar >= nobs then
   error('too many exogenous variables')
end
 
// truncate 1st observation to feed the lag
xlag = grocer_x(1:nobs-1,:);
ylag = grocer_y(1:nobs-1,1);
yt = grocer_y(2:nobs,1);
xt = grocer_x(2:nobs,:);
 
// setup storage for iteration results
iterout = zeros(grocer_maxit,3);
 
while (converg>grocer_crit)&(iter<grocer_maxit) then
  // step 1, using intial rho = 0, do OLS to get bhat
  ystar = yt-rho*ylag;
  xstar = xt-rho*xlag;
  e = grocer_y-grocer_x*ols0(ystar,xstar);
   
  et = e(2:nobs,1);
  elagt = e(1:nobs-1,1);
   
  // step 2, update estimate of rho using residuals
  //         from step 1
   
  rho_last = rho;
  rho = ols0(et,elagt);
  converg = abs(rho-rho_last);
   
  iterout(iter,1) = rho;
  iterout(iter,2) = converg;
  iterout(iter,3) = iter;
   
  iter = iter+1;
   
end
// end of while loop
 
if iter==grocer_maxit then
  error('olsc did not converge in 100 iterations');
end

rolsc = ols2(ystar,xstar);
rolsc('meth') = 'Cochrane-Orcutt'
 
// after convergence produce a final set of estimates using
// rho-value
ystar = yt-rho*ylag;
xstar = xt-rho*xlag;
 
rolsc(1)($+1) = 'prests'
rolsc(1)($+1) = 'namex'
rolsc(1)($+1) = 'namey'
rolsc('prests')=grocer_prests
rolsc('namex')=grocer_namexos
rolsc('namey')=grocer_namey
if grocer_prests then
   rolsc(1)($+1) = 'bounds'
   rolsc('bounds')=grocer_boundsvarb
end
 
rolsc(1)($+1)='rho'
rolsc('rho')=rho
rolsc(1)($+1)='iter'
rolsc('iter')=iterout(1:iter-1,:)
// compute t-statistic for rho
varrho = (1-rho*rho)/(nobs-2);
trho=rho/sqrt(varrho)
rolsc(1)($+1)='trho'
rolsc('trho')=trho

if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(rols,foutput)
   prtuniv(rolsc,%io(2))
   printsep(%io(2))
   write(%io(2),'estimated rho: '+string(rho))
   write(%io(2),'(Student t: '+string(trho)+')')
end



