function [recm]=ecm(grocer_nlag,varargin)

// PURPOSE: performs error correction model estimation
// ------------------------------------------------------------
// INPUT:
// * grocer_nlag = the lag length
// * varargin = a string which can be
//   . 'endo=var1;var2;...;varn' or 'endo=[var1;var2;...;varn]'
//     with vari the name of an admissible variable for explox
//     (a real vector, a ts, a real matrix or a list of such
//     variables)
//   . 'jres=xx' where xx is the name of a johansen results 
//               tlist (optional: if not given, is estimated by
//               the function)
//   . 'nbr=xx' where xx is the # of cointegration vectors to
//     keep from the johansen estimation (optional: if not
//     given, is calculated by the function with a level equal 
//     to plevel)
//   . 'plevel=xx' where xx=0.01, 0.05 or 0.1 is the
//     significance level for the cointegrating vectors 
//     (optional: if not given, is set to 0.05; useless if the
//     option 'nbr=xx' is used)
// ------------------------------------------------------------
// OUTPUT: 
// recm = a recm tlist with:
//   . recm('meth')  = 'ecm
//   . recm('y')     = y data vector
//   . recm('x')     = x data matrix
//   . recm('nobs')  = # observations
//   . recm('nvar')  = # exogenous variables
//   . recm('neqs')  = # endogenous variables
//   . recm('resid') = residuals, with recm('resid')(:,i): 
//                     residuals for equation # i
//   . recm('beta')  = bhat, with recm('beta')(:,i):  
//                     coefficients for equation # i
//   . recm('rsqr')  = rsquared, with recm('rsqr')(i) :
//                     rsquared for equation # i
//   . recm('overallf') = F-stat for the nullity of
//                     coefficients other than the constant
//                     with: recm('f')(i): F-stat for equation
//                     # i
//   . recm('pvaluef') = their significance level with:
//                     recm('pvaluef')(i): significance level
//                     for equation # i
//   . recm('rbar')  = rbar-squared
//   . recm('sigu')  = sums of squared residuals with 
//                     recm('sigu')(:,i): sum of squared 
//                     residuals for equation # i
//   . recm('ser')   = standard errors of the regression with
//                    recm('ser')(i): standard error for
//                    equation # i
//   . recm('tstat') = t-stats, with recm('tstat')(:,i):  
//                     t-stat for equation # i
//   . recm('pvalue')= pvalue of the betas, with
//                      recm('pvalue')(:,i): p-value for
//                      equation # i
//   . recm('dw')    = Durbin-Watson Statistic, with:
//                    recm('dw')(i): DW for equation # i
//   . recm('condindex') = multicolinearity cond index, with
//                         recm('condindex')(i): cond index for
//                         equation # i
//   . recm('boxq') = Box Q-stat, with recm('boxq')(i):
//                    Box Q-stat for equation # i
//   . recm('sigma') = (neqs x neqs) var-covar matrix of the 
//                     regression
//   . recm('aic') = Akake information criterion
//   . recm('bic') = Schwartz information criterion
//   . recm('hq') = Hannan-Quinn information criterion
//   . recm('namey') = name of the y variable
//   . recm('nx') = # of x variables 
//   . recm('namex') = name of the cointegration relations 
//     (if any)
//   . recm('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . recm('nb_coint_relat') = # of cointegration relations
//   . recm('jres') = results of johansen estimation
//   . recm('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
// ------------------------------------------------------------
// NOTES: 
// * constant vector automatically included
// * x-matrix of exogenous variables not allowed
// * error correction variables are automatically
//         constructed using output from Johansen's ML-estimator 
// ---------------------------------------------------
// Copyright Eric Dubois 2002
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jpl@jpl.econ.utoledo.edu
 
// set defaults
grocer_plevel=0.05
grocer_prt=%t

if exists('grocer_boundsvar') then
   // get the bounds p times back, because the y variable in var1  
   // is lead p times
   // transfer the boundsvar variable in the function 
   // so that it can be transformed in the function
   if size(grocer_boundsvar,1) > 2 then
      error('you cannot use discountinous bounds in var')
   end
   if grocer_boundsvar ~= [] then
      grocer_boundsvar=[num2date(date2num(grocer_boundsvar(1))-grocer_nlag,...
                   date2fq(grocer_boundsvar(1))) ; grocer_boundsvar(2)]
   end
end

// separates from the list of variable arguments the list of endogenous variables
// and the options
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      grocer_st=strsubst(varargin(grocer_i),' ','')
      str3=part(grocer_st,1:3)
      str4=part(grocer_st,1:4)
      str6=part(grocer_st,1:6)
      if str4 == 'jres' | str3 == 'nbr' | str6 == 'plevel' then
         execstr('grocer_'+varargin(grocer_i))
      elseif grocer_st == 'noprint' then
          grocer_prt=%f
      elseif part(grocer_st,1:5) == 'endo=' then
         grocer_endo=str2vec(varargin(grocer_i))
      else
         error(typeof(varargin(grocer_i)):'+not a valid option in ecm')
      end
   else
      error(typeof(varargin(grocer_i))+': not a valid type in ecm')
   end
end

// explodes the list of the arguments into the corresponding  
// variable, its name, and, if necessary updates the bounds
[y,namey,grocer_boundsvarb,prests]=explox(grocer_endo,'endogenous')

[nobs,neqs] = size(y);
nameyf='del('+namey+')'

if ~exists('grocer_jres','local') then
   // the user has provided no johansen recm
   if grocer_prt then
      ch='johansen(0,grocer_nlag,'''+joinstr(namey,''',''')+''')'
   else 
      ch='johansen(0,grocer_nlag,'''+joinstr(namey,''',''')+''',''noprint'')'
   end
   grocer_jres = evstr(ch)
end

ecvectors = grocer_jres('evec');
index = grocer_jres('ind');

if ~exists('grocer_nbr','local') then
   // the user has not provided the # of cointegrating vectors
   select grocer_plevel
   case 0.01 then coltsig=3
   case 0.05 then coltsig=2
   case 0.1 then coltsig=1
   else error (string(grocer_plevel)+'is not an admissible value...
        for plevel')
   end
   trstat = grocer_jres('lr1');
   tsignf = grocer_jres('cvt');
   grocer_nbr = 0;
   for i = 1:neqs
      if trstat(i,1)>tsignf(i,coltsig) then
         grocer_nbr = i
      end
   end
end
          
// nvar adjusted for constant term 
k = neqs*grocer_nlag+grocer_nbr+1;
nvar = k;
 
// transform to 1st difference form
dy = tdiff(y,1);
dy = trimr(dy,1,0);
// account for differencing
 
// call VAR using 1st difference and co-integrating variables
// call depends on whether we have an x-matrix or not
if grocer_nbr == 0 then
   recm=var1(dy,grocer_nlag)
   recm(1)($+1)='namey'
   recm($+1)=nameyf
else
   // construct r error correction variables
   x = mlag(y(:,index),1)*ecvectors(:,1:grocer_nbr);
   // account for differencing
   x = trimr(x,1,0);
   recm=var1(dy,grocer_nlag,x)
   recm(1)($+1)='namey'
   recm($+1)=nameyf
   recm(1)($+1)='namex'
   recm($+1)='lag 1 of coint. vec. #'+string([1:grocer_nbr])
end
recm(1)($+1)='prests'
recm($+1)=prests
recm('meth')='ecm'
recm(1)($+1)='nb_coint_relat'
recm('nb_coint_relat')=grocer_nbr
recm(1)($+1)='nx'
recm('nx')=grocer_nbr
recm(1)($+1)='jres'
recm('jres')=grocer_jres

if prests then
   recm(1)($+1)='bounds'
   recm($+1)=[num2date(date2num(grocer_boundsvarb(1))+grocer_nlag,...
              date2fq(grocer_boundsvarb(1))) ; grocer_boundsvarb(2)]
end
 
if grocer_prt then 
   prtjohvec(grocer_jres,grocer_nbr)
   prtvar(recm,%io(2))
end
 
