function [irf_low,irf_upp]=irf_mc1(res,mres,S,N,p,niter,siz)

// PURPOSE: Calculates standard error of Impulse Response 
// Function for VAR by means of Monte-Carlo simulations on
// the coefficient matrix
// ------------------------------------------------------------
// INPUT:
// * res = results tlist returned by VAR
// * mres = decomposition method
// * N = # of endogenous variables in the VAR
// * S = # of periods
// * p = # of lags in the VAR
// * niter = # of draws in the simulation
// * siz = size of the confidence band
//-------------------------------------------------------------
// OUTPUT: 
// * irf_low = ((S+1) x T) lower range of impulse response
//                     confidence band
// * irf_upp = ((S+1) x T) upper range of impulse response
//                     confidence band
//-------------------------------------------------------------
// Copyright Eric Dubois 2002
  
//============================================================
//    SET UP PARMS 
//============================================================

nobs=res('nobs')
betah=res('beta')(1:N*p,:)
sigma=res('sigma')
sqrtom=(sigma^0.5).*.(res('xpxi')^0.5)
dup=duplication(N)
dupp=inv(dup'*dup)*dup'
sqrtsig_sig=(2*dupp*(sigma.*.sigma)*dupp'/nobs)^0.5

sIRF=zeros(niter,(S+1)*N,N)
irf_low=ones(N*(S+1),N)
irf_upp=ones(N*(S+1),N)

i=0
while i < niter
   i=i+1
// draw the betas
   u=grand(N*(N*p+1),1,'nor',0,1)
   drawi=matrix(sqrtom*u,N*p+1,N)
   sbetah=betah+drawi(1:N*p,:)

   select mres
   case 'chol1'
// draw the residual matrix because of the Choleski
// decomposition
      v=matrix(dup*sqrtsig_sig*grand(N*(N+1)/2,1,'nor',0,1),N,N)
      ssigma=sigma+v
      if and(spec(ssigma) > 0) then
      //ssigma is definite positive 
         P=chol(ssigma)'
         m=irf0(sbetah,S,N,p,P)
         sIRF(i,:,:)=m
      else
         i=i-1
      end

   case 'chol2'
// draw the residual matrix because of the Choleski
// decomposition
      v=matrix(dup*sqrtsig_sig*grand(N*(N+1)/2,1,'nor',0,1),N,N)
      ssigma=sigma+v
      if and(spec(ssigma) > 0) then
         //ssigma is definite positive 
         Dsr = diag(sqrt(diag(ssigma)));
         P = chol(ssigma)'*inv(Dsr);
         m=irf0(sbetah,S,N,p,P)
         sIRF(i,:,:)=m
      else
         i=i-1
      end

   case 'original'
      P=eye(N,N)   
      m=irf0(sbetah,S,N,p,P)
      sIRF(i,:,:)=m
   end

end

for j=0:S
   for k=1:N
      for l=1:N
         irf_sort=sort(real(sIRF(:,N*j+l,k)))
         irf_low(N*j+l,k)=irf_sort(floor((1-siz)*niter))
         irf_upp(N*j+l,k)=irf_sort(floor(siz*niter))
      end
   end
end  
