function [rvar]=var(grocer_p,varargin)

// PURPOSE: performs vector autogressive estimation
// ------------------------------------------------------------
// INPUT:
// * grocer_p = the lag length
// * varargin = arguments which can be:
//   . 'endo=[var1 var2 ... varn]' with vari: the ith 
//     endogenous variable in the var
//   . 'exo=[var1 var2 ... varn]' with vari: the ith 
//     exogenous variable in the var
//   . the string 'noprint' if the user doesn't want the 
//     to print the results of the regression
// ------------------------------------------------------------
// OUTPUT: 
// rvar = a results tlist with:
//   . rvar('meth')  = 'var'
//   . rvar('y')     = y data vector
//   . rvar('x')     = x data matrix
//   . rvar('nobs')  = # observations
//   . rvar('nvar')  = # exogenous variables
//   . rvar('neqs')  = # endogenous variables
//   . rvar('resid') = residuals, with rvar('resid')(:,i): 
//                     residuals for equation # i
//   . rvar('beta')  = bhat, with rvar('beta')(:,i):  
//                     coefficients for equation # i
//   . rvar('rsqr')  = rsquared, with rvar('rsqr')(i) :
//                     rsquared for equation # i
//   . rvar('overallf') = F-stat for the nullity of
//                     coefficients other than the constant
//                     with: rvar('f')(i): F-stat for equation
//                     # i
//   . rvar('pvaluef') = their significance level with:
//                     rvar('pvaluef')(i): significance level
//                     for equation # i
//   . rvar('rbar')  = rbar-squared
//   . rvar('sigu')  = sums of squared residuals with 
//                     rvar('sigu')(:,i): sum of squared 
//                     residuals for equation # i
//   . rvar('ser')   = standard errors of the regression with
//                    rvar('ser')(i): standard error for
//                    equation # i
//   . rvar('tstat') = t-stats, with rvar('tstat')(:,i):  
//                     t-stat for equation # i
//   . rvar('pvalue')= pvalue of the betas, with
//                      rvar('pvalue')(:,i): p-value for
//                      equation # i
//   . rvar('dw')    = Durbin-Watson Statistic, with:
//                    rvar('dw')(i): DW for equation # i
//   . rvar('condindex') = multicolinearity cond index, with
//                         rvar('condindex')(i): cond index for
//                         equation # i
//   . rvar('boxq') = Box Q-stat, with rvar('boxq')(i):
//                    Box Q-stat for equation # i
//   . rvar('sigma') = (neqs x neqs) var-covar matrix of the 
//                     regression
//   . rvar('aic') = Akake information criterion
//   . rvar('bic') = Schwartz information criterion
//   . rvar('hq') = Hannan-Quinn information criterion
//   . rvar('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rvar('namey') = name of the y variable
//   . rvar('nx') = # of x variables 
//   . rvar('namex') = name of the x variables (if any)
//   . rvar('bounds') = if there is a timeseries in the 
//     regression, the bounds of the regression
// ------------------------------------------------------------
// Copyright Eric Dubois 2002

grocer_y=[]
grocer_nx=0
grocer_prt=%t
grocer_flagexo=%f

if exists('grocer_boundsvar') then
   // get the bounds p times back, because the y variable in var1  
   // is lead p times
   // transfer the boundsvar variable in the function 
   // so that it can be transformed in the function
   if size(grocer_boundsvar,1) > 2 then
      error('you cannot use discountinous bounds in var')
   end
   if grocer_boundsvar ~= [] then
      grocer_boundsvar=[num2date(date2num(grocer_boundsvar(1))-grocer_p,...
                   date2fq(grocer_boundsvar(1))) ; grocer_boundsvar(2)]
   end
end

grocer_nargin=length(varargin)

for grocer_i=1:grocer_nargin

   if typeof(varargin(grocer_i)) == 'string' then
      grocer_st=strsubst(varargin(grocer_i),' ','')
      if part(grocer_st,1:5) == 'endo=' then
         grocer_endo=str2vec(grocer_st)
      
      elseif part(grocer_st,1:4) == 'exo=' then
         grocer_exo=str2vec(grocer_st)
         grocer_flagexo=%t
      
      elseif grocer_st == 'noprint' then
         grocer_prt=%f
      end
   else
      error(typeof(varargin(grocer_i))+': not a valid type in ecm')
   end

end

// explodes the list of the arguments into the corresponding  
// variables, their names, and, if necessary creates the bounds
[grocer_y,grocer_namey,grocer_boundsvarb,grocer_prests]=...
explox(grocer_endo,'endogenous')

if grocer_flagexo then
   // explodes the list of the arguments into the corresponding  
   // variables, their names, and, if necessary updates the bounds
   [x,namex,grocer_boundsvarb,grocer_prests]=explox(grocer_exo,...
   'exogenous',grocer_boundsvarb)
   grocer_nx=size(x,2)
   rvar=var1(grocer_y,grocer_p,x)
   rvar(1)($+1)='namey'
   rvar($+1)=grocer_namey
   rvar(1)($+1)='namex'
   rvar($+1)=namex
else
   rvar=var1(grocer_y,grocer_p)
   rvar(1)($+1)='namey'
   rvar($+1)=grocer_namey
end
rvar(1)($+1)='prests'
rvar($+1)=grocer_prests
rvar(1)($+1)='nx'
rvar('nx')=grocer_nx

if grocer_prests then
   rvar(1)($+1)='bounds'
   rvar($+1)=[num2date(date2num(grocer_boundsvarb(1))+grocer_p,...
              date2fq(grocer_boundsvarb(1))) ; grocer_boundsvarb(2)]
end

if grocer_prt then 
   prtvar(rvar,%io(2))
end
