function [rvarf]=varf(namevar,hprev,varargin)

// PURPOSE: performs forecasting by the mean of a vector
// autogressive estimation
// ------------------------------------------------------------
// INPUT:
// * namevar = the tlist results resulting from a VAR, an ECM, 
//  a BVAR or a BECM estimation or the same between quotes (to
//  have it's name saved)
// * hprev = the prevision period which can be either
//   . a [n1 n2] constant vector where n1 and n2 are the lead  
//     over the last period of the estimation (n1<=0 means that
//     the forecast begins within the estimation period)
//   . a n constant which is equivalent to [1 n] (forecast
//     begins just after the estimation period)
//   . a [n1 n2] string vector where n1 and n2 are the time
//     periods for forecasting (a posibility open only if the 
//     VAR has been performed with ts)
//   . a n string which is equivalent to [1 n] (forecast
//     begins just after the estimation period)
// * varargin 
//   . 'xp=xx' (necessary if the variables in the VAR has been
//     given as a matrix) where xx is [# of forecasting dates ; 
//      # of exogenous variables in the var] for the values of
//      exogenous variables over the forecasting period
//   . the string 'noprint' if the user doesn't want to
//     print the results of the forecast
// ------------------------------------------------------------
// OUTPUT: 
// rvarf = a results tlist with:
//   . rvarf('meth')  = 'rvarf'
//   . rvarf('rvar')  = results tlist of the originating var
//   . rvarf('prev')  = matrix of forecasts
//   . rvar('prev_namex')  = vector or ts of forecasts called
//     by their names preceded by 'prev_'
// ------------------------------------------------------------
// NOTE: if the var structure comes from an ecm estimation,
// then what is stored is the forecasted levels; if it comes
// from an ordinary var, then what is stored is the forecasted
// values of the endogenous variables, as they have been given
// by the user (for instance, if she has given one as delts(ts),
// then what is forecasted is delts(ts), not ts) 
// ------------------------------------------------------------
// Copyright Eric Dubois 2002

prt=%t
for i=1:length(varargin)
   str2=part(varargin(i),1:2)
   if str2 == 'xp' then
      execstr(varargin(i))
   end
   if varargin(i) == 'noprint' then
      prt=%f
   end
end

// give a name to the VAR and store the VAr itself
select type(namevar)
case 10 then
   rvar=evstr(namevar)
case 16 then
   rvar=namevar
   namevar=emptystr()
end

// retrieves useful parameters from the structures
nlag=rvar('nlag')
nobs=rvar('nobs')+nlag
neqs=rvar('neqs')
prests=rvar('prests')
bet=rvar('beta')
namey=rvar('namey')
meth=rvar('meth')
ecmflag=%f

nbr=0
if meth == 'ecm' | meth == 'becm' then 
   jres=rvar('jres')
   namey=jres('namey')
   nbr=rvar('nb_coint_relat')
   if nbr ~= 0 then
      ecmflag=%t
   end 
end
// for forecasting, it does not matter whether the var is
// bayesian or not

// calculate the range for forecasting
select typeof(hprev)

case 'constant'
   [nl,nc]=size(hprev)
   if nl == 1 & nc == 1 then
      nf=hprev
      n0=1
      hprev=[1 ; hprev]
   else
      if nc ~= 1  & nl ~= 1 then
         error('arg # 2 should be a vector or a constant')
      else
         if max(nc,nl) ~= 2 then
            error('arg # 2 should be a (2x1) or a (1x2) ...
                  vector or a constant')
         else
            n0=hprev(1)
            nf=hprev(2)
         end
      end
   end
   if prests then
   // calculate hprev as a matrix of dates
      b=rvar('bounds')
      bf=b(max(size(b)))
      fq=date2fq(bf)
      hprev=[num2date(date2num(bf)+n0,fq) ;...
             num2date(date2num(bf)+nf,fq) ]
   end

case 'string' then
   [nl,nc]=size(hprev)
   if nl == 1 & nc == 1 then
      n0=1
      b=rvar('bounds')
      bf=b($)
      nf=diff_date(hprev,bf)
      fq=date2fq(bf)
      hprev=[num2date(date2num(bf)+1,fq) ; hprev]
   else
      if nc ~= 1  & nl ~= 1 then
         error('arg # 2 should be a vector or a constant')
      else
         if max(nc,nl) ~= 2 then
            error('arg # 2 should be a (2x1) or a (1x2) ...
                  vector or a constant')
         else
            b=rvar('bounds')
            bf=b(max(size(b)))
            n0=diff_date(hprev(1),bf)
            nf=diff_date(hprev(2),bf)
         end
      end
   end
else
   error(typeof(hprev)+' is not a valid type for arg # 2')
end

if ecmflag then
   evec=jres('evec')
   yvec=jres('y')
   ny=size(yvec,1)
end

if n0 <= 1 & n0 > -nobs then
   yext=rvar('y')(nobs+n0-1:-1:nobs-nlag+n0,:)
   if ecmflag then
      lyp=yvec(ny+n0-1,:)
   end
else 
// forecasting begins some period after the estimation period
// a possibility open only for ts
// then use the underlying data whose name has been stored 
// in the var tlist
   yext=zeros(nlag,neqs)
   lyp=zeros(1,neqs)
// first adjust the bounds to feed the first lags
   fq=date2fq(hprev(1))
   b1=num2date(date2num(hprev(1))-nlag,fq)
   b2=num2date(date2num(hprev(1))-1,fq)
   for i=1:neqs
      val=ts2vec(evstr(rvar('namey')(i)),[b1 ; b2])
      yext(:,i)=val(nlag:-1:1)
   end
   if ecmflag then
      for i=1:neqs
         lyp(i) = values(evstr(jres('namey')(i)),b2)
      end
   end
end

yp=zeros(nf-n0+1,neqs)
xp=[]
vecm=[]
// add exogenous variables for forecasting
nx=rvar('nx')-nbr
if ecmflag then
   vecm=(lyp*evec(:,1:nbr))
   iyext=[lyp ; ones(nf-n0+1,neqs)]
end

if nx ~= 0 then
   if ~exists('xp','local') then
      for i=1:nx
         t=typeof(rvar('namex')(i))
         if t ~= 'ts' then
            error('your var model contains exogenous ...
            variables which are not ts and for which ...
            you have provided no values')
         end
      end
      for i=1:nx
         xp=[xp ts2vec(evstr(rvar('namex')(1)),hprev)]
      end
   end
end

// start forecasting
for i=n0:nf
   if ecmflag then      
      yp0=[matrix(yext',1,nlag*neqs) vecm xp(i) 1]*bet
      iyext(i-n0+2,:)=iyext(i-n0+1,:)+yp0 
      vecm=(iyext(i-n0+2,:)*evec(:,1:nbr))
   else
      yp0=[matrix(yext',1,nlag*neqs) xp(i) 1]*bet
   end
   yp(i-n0+1,:)=yp0
   yext(nlag,:)=[]
   yext=[yp0 ; yext]
end

if ecmflag then      
   yp=iyext(2:nf-n0+2,:)
end

rvarf=tlist(['results';'meth';'namevar';'rvar';'hprev';'prev'],...
'varf',namevar,rvar,hprev,yp)

for i=1:neqs
   if prests then
      aux=reshape(yp(:,i),hprev(1))
   else
      aux=yp(:,i)
   end
   // remove all signs which can be interpreted as operations
   nam=namey(i)
   nam=strsubst(nam,'(','')
   nam=strsubst(nam,')','')
   nam=strsubst(nam,'+','_p_')
   nam=strsubst(nam,'-','_s_')
   nam=strsubst(nam,'*','_m_')
   nam=strsubst(nam,'/','_d_')
   ch='prev_'+nam+'=aux'
   execstr(ch)
   rvarf(1)($+1)='prev_'+nam
   ch='prev_'+nam
   rvarf($+1)=evstr(ch)
end

if prt then
   prtvarf(rvarf,%io(2))
end
