function [F,dF,A,dA,V,dV,G,dG]=arma_dv(theta,theta2mat,di)

// PURPOSE: Computes the partial derivatives of the matrices
// in reduced form of a VARMAX model with respect to i-th
// parameter of the theta vector.
// ------------------------------------------------------------
// INPUT:
// * theta = (npx1) vector of starting values for the
//   parameters
// * theta2mat = a string vector of instructions that
//   transforms theta into the matrices FR, FS, AR, AS, V and G
// * di = a scalar
// ------------------------------------------------------------
// OUTPUT:
// * F = the whole AR part of the model
// * A = the whole MA part of the model
// * V = the variance of the residuals
// * G = the coefficients of the exogenous variables
// * dF, dA, dV, dG = the derivatives of these matrices with
// respect to the i-th parameter in theta
// ------------------------------------------------------------
// NOTE:
// This function does not work with composite models.
// ------------------------------------------------------------
// Copyright Jaime Terceiro, 1997/
// Eric Dubois 2003 for the Scilab translation

if di<1|di>size(theta,1) then
  error('i inconsistent with THETA (out of range)');
end
 
[FR,FS,AR,AS,V,G0] = theta2arm2(theta,theta2mat,1);
dtheta = zeros(size(theta,1),1);
dtheta(di) = 1
[dFR,dFS,dAR,dAS,dV,dG0] = theta2arm2(dtheta,theta2mat,1);
 
F = eye(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_m);
A = eye(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_m);
G = zeros(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_r);
dF = zeros(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_m);
dA = zeros(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_m);
dG = zeros(grocer_e4_m,(1+grocer_e4_k)*grocer_e4_r);
 
if grocer_e4_r then
  G(:,1:grocer_e4_r*(grocer_e4_g+1)) = G0;
  dG(:,1:grocer_e4_r*(grocer_e4_g+1)) = dG0;
end

F(:,grocer_e4_m+1:grocer_e4_m*(grocer_e4_p+1)) = FR;
dF(:,grocer_e4_m+1:grocer_e4_m*(grocer_e4_p+1)) = dFR;
for i = 1:grocer_e4_P
  F(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m) = ...
    F(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m)+FS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
  dF(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m) = ...
    dF(:,i*s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m)+dFS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
   
  for j = 1:grocer_e4_p
    F(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m) = ...
      F(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m)+...
      FR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*FS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
    dF(:,(i*grocer_e4_s+j)*m+1:(i*grocer_e4_s+j+1)*grocer_e4_m) = ...
      dF(:,(i*s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m)+...
      dFR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*FS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m)+...
      FR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*dFS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
  end
end
 
A(:,grocer_e4_m+1:grocer_e4_m*(grocer_e4_q+1)) = AR;
dA(:,grocer_e4_m+1:grocer_e4_m*(grocer_e4_q+1)) = dAR;
for i = 1:grocer_e4_Q
  A(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m) = ...
  A(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m)+AS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
  dA(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m) = ...
  dA(:,i*grocer_e4_s*grocer_e4_m+1:(i*grocer_e4_s+1)*grocer_e4_m)+dAS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
  for j = 1:grocer_e4_q
    A(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m) = ...
      A(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m)+...
      AR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*AS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
    dA(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m) = ...
      dA(:,(i*grocer_e4_s+j)*grocer_e4_m+1:(i*grocer_e4_s+j+1)*grocer_e4_m)+...
      dAR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*AS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m)+...
      AR(:,(j-1)*grocer_e4_m+1:j*grocer_e4_m)*dAS(:,(i-1)*grocer_e4_m+1:i*grocer_e4_m);
  end
end
 
if ~E4OPTION(5) then
  if min(spec(V))<=0 then
    Vu = cholp(V);
    V = Vu'*Vu;
  end
else
  dV = dV*V'+V*dV';
  V = V*V';
end
