function result=varma(grocer_e4_endo,grocer_AR,grocer_ARS,grocer_MA,grocer_MAS,grocer_v,grocer_s,varargin)

// PURPOSE: estimate a VARMA model using E4 functions
// the ARMA model has the following form:
// AR(L)*ARS(L^s) y = MA(L)*MAS(L^s) e [+G(L)X]
// where L is the lag operator, X is an optional vector of 
// exogenous variables
// ------------------------------------------------------------
// INPUT:
// * grocer_e4_endo = either 
//   - a matrix of strings, each one being the name of a
//   variable
//   - a (Txn) real matrix
//   - a ts
//   - a list of variables
//   each element could be a timeseries, a real vector, 
//   a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
// * grocer_AR = the matrix [] or a (nx(n*p)) matrix 
//   with:
//   - n = # of endogenous variables in grocer_e4_endo
//   - p = # of lags in the AR part of the process 
// * grocer_ARS = the matrix [] or a (nx(n*ps)) matrix 
//   with ps = # of lags in the seasonal AR part of the process 
// * grocer_MA = the matrix [] or a (nx(n*q)) matrix 
//   with: q = # of lags in the AM part of the process 
// * grocer_MAS = the matrix [] or a (nx(n*qs)) matrix 
//   with qs = # of lags in the seasonal MA part of the process 
// * grocer_v = a (nx1) vector if the user wants to impose independence
//   between resisduals or a (nxn) matrix in the other case
// * grocer_s = a scalar representing the order of the seasonality
// ------------------------------------------------------------
// OUTPUT:
// result = a results tlist with:
// - result('meth') ='varma'
// - result('y') = (nobsxnendo) matrix of values for the
//   endogenous variables
// - result('namey') = (nvarx1) vector of names for the
//   endogenous variables
// - result('nobs') = # of observations
// - result('coeff') = (npx1) vector of estimated parameters
// - result('lab') = (npx1) string vector of names for the 
//   estimated parameters
// - result('llike') = log-likelihood of the model
// - result('tstat') = Student's t of the coefficients
// - result('std') = (npx1) Student's t of the coefficients
// - result('corr') = (npxnp) correlation matrix
// - result('AIC') = Akaike information criterion
// - result('BIC') = Schwartz information criterion
// - result('theta2mat') = (npx1) string vector making the
//   transformation of the vector of estimated parameters into
//   the matrices of the problem
// - result('seas') = order of the seasonality
// - result('nexo') = # of exogenous variables in the model
// - result('resid') = (nobsx1) vector of filtered residuals
// ------------------------------------------------------------
// Copyright: Eric Dubois 2004

// set defaults
[E4OPTION]=e4init()
grocer_e4_prt=%t
grocer_e4_delta=sqrt(%eps)

// explodes the list of the arguments into the corresponding  
// variables, their names, and, if necessary updates the bounds
[grocer_e4_y,grocer_e4_namey,grocer_e4_boundsvarb,grocer_e4_prests]=...
explox(grocer_e4_endo,'endogenous')
[grocer_e4_T,grocer_e4_m] = size(grocer_e4_y)

grocer_e4_nargin=length(varargin)
for grocer_e4_i=grocer_e4_nargin:-1:1
   select typeof(varargin(grocer_e4_i))
   case 'string' then
      grocer_e4_varaux=strsubst(varargin(grocer_e4_i),' ','')
      if grocer_e4_varaux == 'noprint' then
         grocer_e4_prt=%f
         varargin(grocer_e4_i)=null()
      elseif part(grocer_e4_varaux,1:5) == 'delta' then
         execstr('grocer_e4_'+grocer_e4_varaux)
         varargin(grocer_e4_i)=null()
      elseif part(grocer_e4_varaux,1:4) == 'exo=' then
         grocer_e4_exo=evstr(part(grocer_e4_varaux,5:length(grocer_e4_varaux)))
         varargin(grocer_e4_i)=null()
      elseif part(grocer_e4_varaux,1:5) == 'Gexo=' then
         grocer_e4_Gexo=evstr(part(grocer_e4_varaux,6:length(grocer_e4_varaux)))
         varargin(grocer_e4_i)=null()
      end
    end
end

if exists('grocer_e4_exo','local') then
   if ~exists('grocer_e4_Gexo','local') then 
      error('Gexo should have been entered along with exo')
   else
      [grocer_e4_exo,grocer_e4_namexo,grocer_e4_boundsvarb,grocer_e4_prests]=...
        explox(grocer_e4_exo,'endogenous',grocer_e4_boundsvarb,grocer_e4_z)
      grocer_e4_r = size(grocer_e4_exo,2)
      [theta,grocer_e4_theta2mat,grocer_e4_V2theta,grocer_e4_lab,grocer_AR,grocer_ARS,grocer_MA,grocer_MAS,G,grocer_V,...
      grocer_e4_p,grocer_e4_P,grocer_e4_q,grocer_e4_Q,grocer_e4_g,...
      grocer_e4_s,grocer_e4_k,grocer_e4_n,grocer_e4_np,grocer_e4_type,grocer_e4_vdiag,grocer_e4_ineq] ...
      = arma2param(grocer_e4_m,grocer_AR,grocer_ARS,grocer_MA,grocer_MAS,...
          grocer_v,grocer_s,grocer_e4_Gexo,grocer_e4_r,grocer_e4_namexo);
      grocer_e4_z=[grocer_e4_z  grocer_e4_exo]
   end
else
   grocer_e4_r = 0
   [theta,grocer_e4_theta2mat,grocer_e4_V2theta,grocer_e4_lab,grocer_AR,grocer_ARS,grocer_MA,grocer_MAS,G,grocer_V,...
   grocer_e4_p,grocer_e4_P,grocer_e4_q,grocer_e4_Q,grocer_e4_g,...
   grocer_e4_s,grocer_e4_k,grocer_e4_n,grocer_e4_np,grocer_e4_type,grocer_e4_vdiag,grocer_e4_ineq] ...
    = arma2param(grocer_e4_m,grocer_AR,grocer_ARS,grocer_MA,grocer_MAS,grocer_v,grocer_s);
end
E4OPTION=sete4opt(E4OPTION,'vcond','lyap','econd','zero','var','fac');

grocer_yscale=min(round(log10(mean(abs(grocer_e4_y),'r'))))
grocer_e4_z=grocer_e4_y
grocer_e4_userflag=0
grocer_e4_innov=1
grocer_e4_type2=int(grocer_e4_type/2)
grocer_e4_scaleb = E4OPTION(2);
grocer_e4_vcond = E4OPTION(3);
grocer_e4_econd = E4OPTION(4);
grocer_e4_zeps = E4OPTION(15)*10^(-grocer_yscale)
grocer_e4_deps = .00000001;


if (grocer_e4_econd==2) then
  grocer_e4_MV = 1;
else
  grocer_e4_MV = 0;
end

grocer_e4_filtk = 0;
if E4OPTION(1)==1 then
  grocer_e4_filtk = 1;
end

if grocer_e4_econd==5 then
  if grocer_e4_r then
    grocer_e4_econd = 2;
  else
    grocer_e4_econd = 3;
  end
end

// give good starting values
theta = e4prees1(theta,grocer_e4_theta2mat,grocer_e4_z);

grocer_e4_theta2mat2=''''+grocer_e4_theta2mat+''''
grocer_g=zeros(grocer_e4_np,1)

// create the function that gathers the estimate of the 
// likelihood function and its numerical derivative
grocer_e4_str2f='deff(''[f,grocer_g,ind]=grocer_e4_lffast(p,ind)'',...
   [''f=lffast(p,grocer_e4_theta2mat,grocer_e4_z)'
for i=1:grocer_e4_np
   aux=grocer_e4_delta*[zeros(i-1,1);1;zeros(grocer_e4_np-i,1)]
   grocer_e4_str2f=grocer_e4_str2f+...
   ''';''grocer_g('+string(i)+')=(lffast(p+['...
   +joinstr(string(aux),';')+'],grocer_e4_theta2mat,grocer_e4_z)-...
   lffast(p-['+joinstr(string(aux),';')+'],grocer_e4_theta2mat,grocer_e4_z))/2/'+string(grocer_e4_delta)
end
grocer_e4_str2f=grocer_e4_str2f+'''])'
execstr(grocer_e4_str2f)

sqrteps=sqrt(%eps)
//sqrteps=1e-3

if or(grocer_e4_ineq(:,1) ~= -%inf) | or(grocer_e4_ineq(:,2) ~= %inf) then
// there are inequality constraints
   [f,theta,g] = optim(grocer_e4_lffast,'b',grocer_e4_ineq(:,1),grocer_e4_ineq(:,2),theta);
   if g'*g > sqrteps then
// numerical estimation has failed to provide acurate
// estimate; then, use the exact derivative 
      [f,theta,g] = optim(lfmod_gmod_103,'b',grocer_e4_ineq(:,1),grocer_e4_ineq(:,2),theta);
   end
else
// numerical estimation has failed to provide acurate
// estimate; then, use the exact derivative 
   [f,theta,g] = optim(grocer_e4_lffast,theta);
   [f,g]=lfmod_gmod_103(theta)
   if g'*g > sqrt(%eps) then
      [f,theta,g] = optim(lfmod_gmod_103,theta);
   end
end

[f,z1,x0]=lffast(theta,grocer_e4_theta2mat,grocer_e4_z)
[std,corrm,varm,Im] = imod(theta,grocer_e4_theta2mat,grocer_e4_z);
tstat=theta ./ std
AIC = 2*(f+grocer_e4_np)/grocer_e4_T
BIC = (2*f+grocer_e4_np*log(grocer_e4_T))/grocer_e4_T

result=tlist(['results';'meth';'y';'namey';'nobs';'nendo';'nvar';'coeff';'lab';'like';'grad';...
'tstat';'std';'exact';'cov';'AIC';'BIC';'theta2mat';'seas';'nexo';'resid'],'varma',grocer_e4_y,grocer_e4_namey,...
grocer_e4_T,grocer_e4_m,...
grocer_e4_np,theta,grocer_e4_lab,...
-f,-g,tstat,std,%t,corrm,AIC,BIC,grocer_e4_theta2mat,grocer_e4_s,grocer_e4_r,z1)

if exists('grocer_e4_exo','local') then
   result(1)($+1)='exo'
   result('exo')=grocer_e4_exo
   result(1)($+1)='namexos'
   result('namexos')=grocer_e4_namexo
   result(1)($+1)='lagexo'
   result('lagexo')=grocer_e4_g
end  

if grocer_e4_prt then
   prtvarma(result);
end
